package d64;


public class BAM {
	
	D64 d64;

	BAM(D64 d64) {

		this.d64 = d64;						
	}
	
	public int diskFreeBlocksCount() {
		
		int blocksFree = 0;
		
		for (int track = 1; track <= 35; track++) {

			if(track <= 35 && track != 18) blocksFree += freeSectorsInTrack(track);			
		}
		
		return blocksFree;
	}
			
	public int freeSectorsInTrack(int track) {
					
		byte[] bamData = d64.readSector(18, 0); 
				
		int trackOffset = 4 + (track-1) * 4;		// Ignore First 4 bytes of the BAM Sector (next Track,next Sector, Formattype	"A", DoubleSieded "2"		
				
		int freeSectors = (bamData[trackOffset] & 0xff);
				
		return freeSectors;
	}	
	
	public boolean isBAMSectorFree(int track, int sector) {
		
		byte[] bamData = d64.readSector(18, 0); // Read Bam (18,0)
		
		int bamTrackOffset = 4 + (track-1) * 4;				
		int sectorBamBits = (bamData[bamTrackOffset+1] & 0xff) | (bamData[bamTrackOffset+2] & 0xff) << 8 | (bamData[bamTrackOffset+3] & 0xff) << 16;				
		
		if((sectorBamBits & (1 << sector)) == (1 << sector)) return true;
		
		return false;
	}
	
	public void writeBAMSectorState(int track, int sector, boolean free) {
		
		byte[] bamData = d64.readSector(18, 0);
		
		int trackOffset = 4 + (track-1) * 4;		// Ignore First 4 bytes of the BAM Sector (next Track,next Sector, Formattype	"A", DoubleSieded "2"		
		
		int sectorBamBits = (bamData[trackOffset+1] & 0xff) | (bamData[trackOffset+2] & 0xff) << 8 | (bamData[trackOffset+3] & 0xff) << 16;				
				
		if(free) 	sectorBamBits = sectorBamBits | (1 << sector);
		else			sectorBamBits = sectorBamBits & ~(1 << sector);	
				
		// * count Up (free) or Down (not free) free Sectors in this Track *
		if(free) 	bamData[trackOffset] 	= (byte) ((bamData[trackOffset] & 0xff) +1);
		else			bamData[trackOffset] 	= (byte) ((bamData[trackOffset] & 0xff) -1);
		
		// * Write the State Bits *
		bamData[trackOffset+1] 	= (byte) (sectorBamBits & 0xff);
		bamData[trackOffset+2] 	= (byte) ((sectorBamBits >> 8) & 0xff);
		bamData[trackOffset+3] 	= (byte) ((sectorBamBits >> 16) & 0xff);
				
		d64.writeSector(bamData, 18, 0);		
	}
			

	public int[][] getCompleteBamTrackSectorStates() {
				
		int[][] bamTracksAndSectors = new int[35][21];
		
		int dirOffset = d64.getOffset(18,0) + 4 ; // Track 18, Sektor 0			
		
		dirOffset += 4; // Ignore First 4 bytes of the BAM Sector (next Track,next Sector, Formattype	"A", DoubleSieded "2"
		
		for (int t = 0; t < 35; t++) {

			int BAMBits = (d64.diskData[dirOffset+1] & 0xff) | (d64.diskData[dirOffset+2] & 0xff) << 8 | (d64.diskData[dirOffset+3] & 0xff) << 16;				

			for(int s=0;s<d64.trackSectors[t];s++) {

				if((BAMBits & (1 << s)) == (1 << s)) 	bamTracksAndSectors[t][s] = 1;				
				else	 									bamTracksAndSectors[t][s] = 0;		
			}
			dirOffset +=4;			
		}			
		
		return bamTracksAndSectors;
	}
	
	public void printBAMTrackSectorStates(int[][] bamTracksAndSectors) {
		
		for (byte t = 0; t < 35; t++) {
			
			System.out.print("Track: " + (t+1) + "  ");	
			
			for(int s=0;s<d64.trackSectors[t];s++) System.out.print(bamTracksAndSectors[t][s]);	
			System.out.println();	
		}				
	}	
}
