package d64;

import java.io.ByteArrayOutputStream;
import java.io.RandomAccessFile;
import java.util.ArrayList;

import main.C64Dir;

public class D64 {

	static public int ERROR_CODE_DISK_FULL 	= 1;
	static public int ERROR_CODE_DIR_FULL 	= 2;


	public String diskPath = "";
	public String diskName = "";
	public String diskID = "";

	public byte[] diskData = new byte[174848]; 

	BAM bam;

	C64Dir c64Dir;

	public ArrayList<DirEntry> d64FileList;


	public D64(C64Dir c64Dir, String filePath) {

		this.c64Dir = c64Dir;
		this.diskPath = filePath;

		updateDiskDataArray();

		bam = new BAM(this);

		d64FileList = getDirEntrys();

	}


	public void updateDiskDataArray() {

		try {

			RandomAccessFile completeFile = new RandomAccessFile(diskPath, "r");	
			completeFile.seek(0);
			completeFile.read(diskData);	
			completeFile.close();		
			
			updateDiskNameAndID();
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}


	public boolean fileExist(String name) {
		
		for(int i=0;i<d64FileList.size();i++) {
						
			if(d64FileList.get(i).getFileName().compareTo(name) == 0) return true;
		}
		
		return false;
	}
	
	public void setDiskNameAndID(String diskName, String diskID) {

		if(diskName == null || diskID == null) return;
		
		// Track 18, Sektor 0		
		int track18SectorOffset = getOffset(18,0); 	
					
		byte name[] = new byte[16];
		for(int i=0;i<16;i++) name[i]	= (byte)160;
		System.arraycopy(diskName.getBytes(), 0, name, 0, diskName.length());

		byte id[] = new byte [5];
		for(int i=0;i<5;i++) id[i]	= (byte)160;
		System.arraycopy(diskID.getBytes(), 0, id, 0, diskID.length());
		
		
		try {
			
			RandomAccessFile raf = new RandomAccessFile(diskPath, "rw");		
			raf.seek(track18SectorOffset+144);
			raf.write(name);	
			raf.seek(track18SectorOffset+162);
			raf.write(id);	
			raf.close();
			
			updateDiskDataArray();

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	
	private void updateDiskNameAndID() {

		try {
			
			// Track 18, Sektor 0		
			int track18SectorOffset = getOffset(18,0); 	
					

			byte name[] = new byte[16];
			System.arraycopy(diskData, track18SectorOffset + 144, name, 0, 16);

			byte id[] = new byte [6];
			System.arraycopy(diskData, track18SectorOffset + 162, id, 0, 5);

			String s = "";
			for(int i=0;i<16;i++) {

				if((name[i] & 0xff) == 160) break;
				s += (char)C64_TO_ASCII_TABLE[(name[i] & 0xff)];
			}

			String s2 = "";
			for(int i=0;i<5;i++) {

				s2 += (char)C64_TO_ASCII_TABLE[(id[i] & 0xff)];
			}

			diskName 	= s.trim();
			diskID 		= s2.trim();
		} 
		catch (Exception e) {
			e.printStackTrace();
		}
	}


	public ArrayList<DirEntry> getDirEntrys() {

		//bam.printBAMTrackSectorStates(bam.getCompleteBamTrackSectorStates());

		ArrayList<DirEntry> list = new ArrayList<DirEntry>();		

		int dirOffset = getOffset(18,0); // Track 18, Sektor 0			

		try {

			// * Read the whole Directory *

			int nextTrack 	= 18;
			int nextSektor 	= 1; 
			int dataPointer 	= 0; 

			int thisSektor 	= 1; 

			boolean seekEntries = true;
			while(seekEntries) {

				if(nextTrack != 18) break;  // Wenn keine Entries mehr vorhanden = Ende der Schleife

				// * Jump to next Sektor *					
				dataPointer = dirOffset + nextSektor*256;

				// * Read Sektor *
				for(int b=0;b<256;b+=32) {

					DirEntry dirEntry = new DirEntry();

					// * Save the pure byte data *
					System.arraycopy(diskData, dataPointer, dirEntry.getData(), 0, 32);


					// * Only the first Entry in this Sector has the next DIR Track/Sektor Information *
					if(b == 0) {

						thisSektor = nextSektor;

						nextTrack 					= diskData[dataPointer];
						nextSektor 					= diskData[dataPointer+1];
					}

					dirEntry.setLocatedAtSector(thisSektor); // remember where this Dir Entry is located		
					dirEntry.setLocatedAtSectorPos(b);

					dirEntry.setNextTrack(diskData[dataPointer]);
					dirEntry.setNextSector(diskData[dataPointer+1]);
					dirEntry.setFileType(diskData[dataPointer+2]);		
					dirEntry.setFileFirstTrack(diskData[dataPointer+3]);
					dirEntry.setFileFirstSector(diskData[dataPointer+4]);	

					// * Do NOT show Scratched files (0x00), only 0x8X files *

					if((dirEntry.getFileType() & 0xff) != 0x00) {

						// * Füll-Zeichen im Namen am Ende ignorieren *
						String nameString = "";
						for(int i=0;i<16;i++) {

							if((diskData[dataPointer+5+i] & 0xff) == 160 || (diskData[dataPointer+5+i] & 0xff) == 0) continue;	
							nameString += (char)C64_TO_ASCII_TABLE[(diskData[dataPointer+5+i] & 0xff)];
						}		

						if(nameString.length() == 0) continue;
						dirEntry.setFileName(nameString);

						int loByte =  (diskData[dataPointer+30] & 0xff);
						int hiByte =  (diskData[dataPointer+21] & 0xff) << 8;
						int blockSize = hiByte | loByte;
						dirEntry.setFileBlockSize(blockSize);

						list.add(dirEntry);	// * Add Entry to the list *					
					}

					dataPointer += 32; 
				}	
			}

		} catch (Exception e) {
			e.printStackTrace();
		}


		/*

		System.out.println("-------------");

		for(int i=0;i<list.size();i++) {

			System.out.print(rpad("Located at Track: " + 18, 22));
			System.out.print(rpad("Sector: " + list.get(i).getLocatedAtSector(), 12));
			System.out.print(rpad("Pointer: " + list.get(i).getLocatedAtSectorPos(), 16));
			System.out.print(rpad("Next Track/Sector: " + list.get(i).getNextTrack() + "/" +list.get(i).getNextSector(), 29));
			System.out.print(rpad("Name: " + list.get(i).getFileName(), 25));
			System.out.print(rpad("Type: " + (list.get(i).getFileType() & 0xff), 13));
			System.out.print(rpad("Size: " + list.get(i).getFileBlockSize(), 12));
			System.out.print(rpad("First Track: " + list.get(i).getFileFirstTrack(), 19));
			System.out.println(rpad("First Sector: " + list.get(i).getFileFirstSector(), 22));	
		}

		 */

		return list;
	}

	public String rpad(String inStr, int finalLength) {		
		return (inStr + "                          " ).substring(0, finalLength);
	}


	protected int getOffset(int track, int sector) {

		int offset = 0;

		for(int t=0;t<track-1;t++) {

			int sectors = trackSectors[t];
			offset += sectors*256;			
		}

		offset += sector*256;

		return offset;
	}



	public void deletePRGFromDisk(DirEntry dirEntry) {

		if(dirEntry != null) {

			int nextTrack 	= dirEntry.getFileFirstTrack();
			int nextSector 	= dirEntry.getFileFirstSector();

			byte[] emptyData = new byte[256];

			while(nextTrack >= 1 && nextTrack <= 35) {
				
				if(nextTrack  == 0 || nextTrack > 35) break;		
				
				byte[] data = readSector(nextTrack, nextSector);
				
				writeSector(emptyData, nextTrack, nextSector);
				bam.writeBAMSectorState(nextTrack, nextSector, true);

				nextTrack 	= data[0] & 0xff;
				nextSector 	= data[1] & 0xff;	
				
			}

			// * Mark DirEntry as scrached *
			dirEntry.setFileType((byte)0x00);
			dirEntry.setFileBlockSize((byte)0x00);
			writeDirEntryDataToDisk(dirEntry);

			updateDiskDataArray();
		}		
	}


	public DirEntry loadPRGFromDisk(String filename) {

		DirEntry dirEntry = null;

		for(int i=0;i<d64FileList.size();i++) {

			if(d64FileList.get(i).getFileName().compareTo(filename) == 0) dirEntry = d64FileList.get(i);
		}

		return dirEntry;
	}


	public byte[] getPRGData(DirEntry dirEntry) {

		ByteArrayOutputStream out = new ByteArrayOutputStream();

		if(dirEntry != null) {

			int nextTrack 	= dirEntry.getFileFirstTrack();
			int nextSector 	= dirEntry.getFileFirstSector();

			int len = 0;
			int readLen = 256;

			while(nextTrack >= 1 && nextTrack <= 35) {

				byte[] data = readSector(nextTrack, nextSector);
				nextTrack 	= data[0] & 0xff;
				nextSector 	= data[1] & 0xff;	

				//System.out.println("Read  at: " + nextTrack + "/" + nextSector + ", ");

				if(nextTrack == 0) 	readLen = (nextSector+1)-2;
				else 				readLen = data.length-2;

				out.write(data, 2, readLen);

				len +=  readLen;		

				if(nextTrack  == 0 || nextTrack > 35) break;		
			}

			//System.out.println("len: " + len);
		}

		//System.out.println();

		return out.toByteArray();
	}


	// * Write PRG bytes on Disk *
	public int writePRGData(String fileName, byte[] prgFileData, int fileType) {

		
		if(prgFileData == null) prgFileData = new byte[] {};
		
		if((fileType & 0x03) != 0) {

			// * Enough Blocks free? *
			int prgSectorCount = (int) Math.ceil((double)prgFileData.length / 254);  // Achtung: aufrunden! Sonst fehlt 1 Sector bei krummen zahlen 

			if(freeBlocks() < prgSectorCount) return ERROR_CODE_DISK_FULL;

			int[] trackSector = getNextFreeSector(17, 0);

			int firstTrack 	= trackSector[0];
			int firstSector 	= trackSector[1];

			int track = 0;
			int sector = 0;

			int bytesLeft = prgFileData.length;

			for(int i=0;i<prgSectorCount;i++) {

				if(track != -1 && sector != -1) {

					track = trackSector[0];
					sector = trackSector[1];


					// * Write PRG Data into Sector Data Array at Pos 2 *
					byte[] sectorData = new byte[256];

					if(bytesLeft >=254)  System.arraycopy(prgFileData, i*254, sectorData, 2, 254);
					else {

						sectorData = new byte[bytesLeft+2];
						System.arraycopy(prgFileData, i*254, sectorData, 2, bytesLeft);
					}


					bam.writeBAMSectorState(track, sector, false); // Mark it as not free


					// * get the next free Track/Sector *
					trackSector = getNextFreeSector(trackSector[0], trackSector[1]);		

					if(i == prgSectorCount-1) {

						sectorData[0] = (byte) 0;
						sectorData[1] = (byte) (bytesLeft+1);
					}
					else {

						sectorData[0] = (byte) trackSector[0];
						sectorData[1] = (byte) trackSector[1];
					}

					//System.out.println("Write  Track/Sector: " + track + "/" + sector + "   " + (sectorData[0] & 0xff)  + " " + (sectorData[1] & 0xff));

					writeSector(sectorData, track, sector);	

					bytesLeft -= 254;			

				}	
				else return ERROR_CODE_DISK_FULL;				
			}


			// * Add Directory Entry *
			writeDirEntry(fileName, (byte)fileType, (byte)firstTrack, (byte)firstSector, (byte)prgSectorCount);

		}
		else { // * 0 Blocks DEL Files *

			writeDirEntry(fileName, (byte)0x80, (byte)18, (byte)0, 0);  
		}

		// * Reload the Dir Data *
		updateDiskDataArray();

		//bam.printBAMTrackSectorStates(bam.getCompleteBamTrackSectorStates());

		return 0;
	}



	public boolean writeDirEntry(String filename, byte fileType, byte firstTrack, byte firstSector, int sectorCount) {


		DirEntry firstSectorEntry 	= null;
		DirEntry lastDirEntry 		= null;

		ArrayList<DirEntry> list = getDirEntrys();
		for(int i=0;i<list.size();i++) {

			DirEntry entry 	= list.get(i);
			lastDirEntry 	= entry;

			if(list.get(i).getLocatedAtSectorPos() == 0) firstSectorEntry = entry;

			//*  Is there a File with FileType 0x00 in the Directory? Then take it! *			
			
			if(list.get(i).getFileType() == 0x00) {

				entry.setFileName(filename);
				entry.setFileType(fileType);
				entry.setFileBlockSize(sectorCount & 0xff);
				entry.setFileFirstTrack(firstTrack);
				entry.setFileFirstSector(firstSector);	

				writeDirEntryDataToDisk(entry);

				return true;
			}
		}


		// ... Else: add the rest of Dir Entry Informations (Located at Dir Sector [pos,pointer], Next Track, Next Sector) *

		// * Create New Dir Entry *
		DirEntry dirEntry = new DirEntry();
		dirEntry.setFileName(filename);
		dirEntry.setFileType(fileType);
		dirEntry.setFileBlockSize(sectorCount & 0xff);
		dirEntry.setFileFirstTrack(firstTrack);
		dirEntry.setFileFirstSector(firstSector);	


		// * If this is the first file in the Dir, then write it *
		if(list.size() == 0) {

			dirEntry.setLocatedAtSector(1);		
			dirEntry.setLocatedAtSectorPos((byte)0);

			dirEntry.setNextTrack((byte)0);
			dirEntry.setNextSector((byte)255);

			d64FileList.add(dirEntry);
			writeDirEntryDataToDisk(dirEntry);	

			return true;
		}
		else {

			
			// Check the last Entry						
			if(lastDirEntry != null) {

				if(lastDirEntry.getLocatedAtSectorPos() < 224) {

					dirEntry.setLocatedAtSector(lastDirEntry.getLocatedAtSector());		
					dirEntry.setLocatedAtSectorPos((byte)lastDirEntry.getLocatedAtSectorPos() + 32);

					dirEntry.setNextTrack((byte)0);
					dirEntry.setNextSector((byte)0);

					d64FileList.add(dirEntry);
					writeDirEntryDataToDisk(dirEntry);	

					return true;
				}
				else {

					int nextSector = (lastDirEntry.getLocatedAtSector() & 0xff) + 3;
					if(nextSector > 17) nextSector -= 17;

					// * Change the Frist Track in this sector and write the Next Track/Sector numbers *
					firstSectorEntry.setNextTrack((byte)18);
					firstSectorEntry.setNextSector((byte)nextSector);							
					writeDirEntryDataToDisk(firstSectorEntry);	

					// * Add and write the new Entry *
					dirEntry.setLocatedAtSector(nextSector);		
					dirEntry.setLocatedAtSectorPos((byte)0);
					dirEntry.setNextTrack((byte)0);
					dirEntry.setNextSector((byte)255);

					d64FileList.add(dirEntry);
					writeDirEntryDataToDisk(dirEntry);	

					return true;
				}
			}	
		}

		return false;
	}

	public void writeDirEntryDataToDisk(DirEntry dirEntry) {

		int offset = getOffset(18, dirEntry.getLocatedAtSector());

		try {

			RandomAccessFile raf = new RandomAccessFile(diskPath, "rw");		
			raf.seek((offset + (dirEntry.getLocatedAtSectorPos() & 0xff)));
			raf.write(dirEntry.getData());	
			raf.close();

		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	
	public void writeDiskNameAndIDToDisk(String diskName, String diskID) {

		if(diskName == null || diskID == null) return;
		
		int offset = 91536;

		byte name[] = new byte[18];
		System.arraycopy(diskName.getBytes(), 0, name, 0, diskName.length());

		byte id[] = new byte [6];
		System.arraycopy(diskID.getBytes(), 0, id, 0, diskID.length());
		
		for(int i=0;i<18-diskName.length();i++) 	name[i]	= (byte)160;
		for(int i=0;i<6-diskID.length();i++) 	id[i]	= (byte)160;
		
		try {

			RandomAccessFile raf = new RandomAccessFile(diskPath, "rw");		
			raf.seek(91536);
			raf.write(name);	
			raf.write(id);	
			raf.close();

		} catch (Exception e) {
			e.printStackTrace();
		}
	}



	public int[] getNextFreeSector(int track, int sector) {

		int[] trackSector = {track,sector};


		if(isSectorFree(track, sector) == false) {

			sector += 10;

			while(isSectorFree(track, sector) == false) {

				//System.out.println("Free Sectors: " +  bam.freeSectorsInTrack(track));
				//System.out.println("Not Free: " +  track + "/" + sector);

				sector++;
				if(sector >= trackSectors[track-1]) {

					sector = (sector - trackSectors[track-1]-2) ;	// Subtract the number of sectors on the track.
				}


				// * Calculate next Track, if no free Sectors are available on this Track *
				if(bam.freeSectorsInTrack(track) == 0) {

					if(track >= 18) {

						track++;
						if(track > 35) return new int[] {-1,-1};
					}
					else	 {

						track--;
						if(track < 1) track = 19;
					}
				}
			}
		}

		trackSector[0] = track;
		trackSector[1] = sector;

		return trackSector;		
	}

	synchronized public byte[] readSector(int track, int sector) {

		int offset = getOffset(track,sector);

		byte[] sectorData = new byte[256]; 

		try {

			RandomAccessFile raf = new RandomAccessFile(diskPath, "r");		
			raf.seek(offset);
			raf.read(sectorData);	
			raf.close();

		} catch (Exception e) {
			e.printStackTrace();
		}

		return sectorData;
	}

	synchronized public boolean writeSector(byte[] sectorData, int track, int sector) {

		int offset = getOffset(track,sector);

		try {

			RandomAccessFile raf = new RandomAccessFile(diskPath, "rw");		
			raf.seek(offset);
			raf.write(sectorData);	
			raf.close();

			return true;

		} catch (Exception e) {
			e.printStackTrace();
		}

		return false;
	}

	public int freeBlocks() {

		return bam.diskFreeBlocksCount();
	}

	public boolean isSectorFree(int track, int sector) {
		return bam.isBAMSectorFree(track, sector);
	}

	public static String reverseCase(String text)
	{
	    char[] chars = text.toCharArray();
	    for (int i = 0; i < chars.length; i++)
	    {
	        char c = chars[i];
	        if (Character.isUpperCase(c))
	        {
	            chars[i] = Character.toLowerCase(c);
	        }
	        else if (Character.isLowerCase(c))
	        {
	            chars[i] = Character.toUpperCase(c);
	        }
	    }
	    return new String(chars);
	}

	
	static public int[] trackSectors = {21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,19,19,19,19,19,19,19,18,18,18,18,18,18,17,17,17,17,17};

	public final static int[] C64_TO_ASCII_TABLE = {  // 0...255

			0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 
			0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,	
			0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f,	
			0x30, 0x31, 0x32, 0x33,	0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3a, 0x3b, 0x3c, 0x3d, 0x3e, 0x3f,
			0x40, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47,	0x48, 0x49, 0x4a, 0x4b, 0x4c, 0x4d, 0x4e, 0x4f,
			0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5a, 0x5b,	0x5c, 0x5d, 0x5e, 0xa4, 
			0x60, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6a, 0x6b, 0x6c, 0x6d, 0x6e, 0x6f,	
			0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7a, 0x7b, 0x7c, 0x7d, 0x7e, 0x7f,	
			0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,	
			0x20, 0x20, 0x20, 0x20,	0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
			0x20, 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7, 0xa8, 0xa9, 0xaa, 0xab, 0xac, 0xad, 0xae, 0xaf,
			0xb0, 0xb1, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7, 0xb8, 0xb9, 0xba, 0xbb, 0xbc, 0xbd, 0xbe, 0xbf,	
			0x60, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6a, 0x6b, 0x6c, 0x6d, 0x6e, 0x6f, 
			0x70, 0x71, 0x72, 0x73,	0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7a, 0x7b, 0x7c, 0x7d, 0x7e, 0x7f,
			0x20, 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7, 0xa8, 0xa9, 0xaa, 0xab, 0xac, 0xad, 0xae, 0xaf,	
			0xb0, 0xb1, 0xb2, 0xb3,	0xb4, 0xb5, 0xb6, 0xb7, 0xb8, 0xb9, 0xba, 0xbb, 0xbc, 0xbd, 0xbe, 0xbf
	};
	
	static public String convertC642ASCII(String str) {
				
		String newStr = "";	
		for(int i=0;i<str.length();i++) newStr += (char)C64_TO_ASCII_TABLE[str.charAt(i)];
				
		return newStr;
	}
	
}
