package d64;

public class DirEntry {

	
	// * DATA Byte Array Infos *
	
	// nextTrack; 			// 00
	// nextSector;			// 01
	// fileType;				// 02    	(0X = Scrached File /  8X [X:   0 = DEL, 1 = SEQ, 2 = PRG, 3 = USR, 4 = REL] )
	// fileFirstTrack;		// 03
	// fileFirstSector;		// 04
	// fileName;				// 05-20  	(Length: 16, empty chars filled with 160)
	// relFirstTrack;		// 21
	// relFirstSector;		// 22
	// relRecordLen;			// 23
							// 24-30 	unused
	// fileBlockSize;		// 30-31 	(lo byte), 31 (hi byte)
	
	private byte[] data = new byte[32];
	
	
	
	private int locatedAtSector; 	
	private int locatedAtSectorPos; 	
	
	public void setLocatedAtSector(int sector) {		
		this.locatedAtSector = sector;
	}
	
	public void setLocatedAtSectorPos(int pointerPos) {		
		this.locatedAtSectorPos = pointerPos;
	}
	
	public int getLocatedAtSector() {
		return locatedAtSector;
	}
	
	public int getLocatedAtSectorPos() {
		return locatedAtSectorPos;
	}
	

	
	public byte[] getData() {
		return data;
	}
	
	public void setData(byte[] data) {
		System.arraycopy(data, 0, this.data, 0, 32);
	}
	
	public int getNextTrack() {		
		return data[0] & 0xff;
	}
	
	public void setNextTrack(byte nextTrack) {
		data[0] = (byte) nextTrack;
	}
	
	public int getNextSector() {		
		return data[1] & 0xff;
	}
	
	public void setNextSector(byte nextSector) {
		data[1] = (byte) nextSector;
	}
	
	public int getFileType() {		
		return data[2];
	}
	
	public void setFileType(byte fileType) {
		data[2] = (byte) (fileType);		
	}
	
	public int getFileFirstTrack() {		
		return data[3] & 0xff;
	}
	
	public void setFileFirstTrack(byte firstTrack) {
		data[3] = (byte) firstTrack;
	}
	
	public int getFileFirstSector() {		
		return data[4] & 0xff;
	}
	
	public void setFileFirstSector(byte firstSector) {
		data[4] = (byte) firstSector;
	}
	
	public String getFileName() {		
		
		// * Füll-Zeichen im Namen am Ende ignorieren *
		String nameString = "";
		for(int i=0;i<16;i++) {

			if((data[5+i] & 0xff) == 160 || (data[5+i] & 0xff) == 0) continue;	
			nameString += (char)D64.C64_TO_ASCII_TABLE[(data[5+i] & 0xff)];
		}		
		
		return nameString;
	}
	
	public void setFileName(String name) {
		
		for(int i=0;i<16;i++) {
			
			if(i<name.length()) 	data[5+i] = (byte)D64.C64_TO_ASCII_TABLE[name.charAt(i)];
			
			// * Füll-Zeichen im Namen am Ende hinzufuegen *
			else 				data[5+i] = (byte)0xA0;				
		}
	}

	public int getFileBlockSize() {
		return ((data[31] << 8) | (data[30] & 0xff));
	}
	
	public void setFileBlockSize(int size) {
		
		data[30] = (byte) (size & 0xff);
		data[31] = (byte) ((size >> 8) & 0xff);		
	}
	
		
	public String getFileTypeString() {

		int fileType = getFileType() & 0x0F;
		
		String typeStr = "";
		if(fileType == 0x00) typeStr = "DEL";
		if(fileType == 0x01) typeStr = "SEQ";
		if(fileType == 0x02) typeStr = "PRG";
		if(fileType == 0x03) typeStr = "USR";
		if(fileType == 0x04) typeStr = "REL";

		return typeStr;
	}		
}
