package ffilechooser;

import java.awt.AWTEvent;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.awt.event.AWTEventListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.font.LineMetrics;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;
import java.awt.image.BufferedImage;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JTextField;
import javax.swing.UIManager;

import ffilechooser.FMonitor.MonitorListener;

public class FButton extends JButton implements MouseListener {

	float x 			= 0;
	float y 			= 0;
	float width 		= 0.02f;
	float height 	= 0.01f;

	String fontName 	= "Arial";
	int fontStyle 	= Font.PLAIN;
	float fontSize 	= 0.01f;
	float radius 	= 0.005f;

	private BufferedImage image;
	private BufferedImage imageResized;

	Color frameColor 		= Color.GRAY;
	Color backgroundColor 	= Color.WHITE;
	Color fontColor 			= Color.BLACK;

	Color backgroundColorEnabled 	= Color.WHITE;
	Color fontColorEnabled 			= Color.BLACK;
	Color backgroundColorDisabled 	= new Color(240,240,240);
	Color fontColorDisabled 			= new Color(140,140,140);
	
	boolean pressed = false;
	
	String text = "";


	float monitorOldWidth = 0;
	float monitorOldHeight = 0;
	
	
	int borderSpaceWidth = 2;
	int borderSpaceheight = 2;
	
	
	FFrame fframe;


	public FButton(final FFrame fframe) {

		this.fframe = fframe;

		setBackground(new Color(0x0, true)); //Nimbus
		setBorder(BorderFactory.createEmptyBorder(6, 10, 4, 8));


		addMouseListener(this);
				
		Toolkit.getDefaultToolkit().addAWTEventListener(new AWTEventListener() {

			@Override
			public void eventDispatched(AWTEvent event) {

				if(fframe.monitor.width != monitorOldWidth || fframe.monitor.height != monitorOldHeight) {

					monitorOldWidth = fframe.monitor.width;
					monitorOldHeight = fframe.monitor.height;
					updateBounds();
					updateFont();		
				}
			}
		}, AWTEvent.PAINT_EVENT_MASK);
				
	}

	public void setText(String s) {
		
		this.text = s;
		
		super.setText("");		
	}	


	public void setImage(BufferedImage image) {		

		this.image 	= image;
		updateImage();
	}

	private void updateImage() {

		resizeImage();
		repaint();		
	}

	public void setFont(Font font) {				
		super.setFont(font);

		if(fframe == null) return;

		fontName 	= font.getName();
		fontStyle 	= font.getStyle();
		fontSize 	= fframe.monitor.convertRealToFloatHeight(font.getSize());
	}	

	public void setFontSize(float size) {

		this.fontSize = size;
		updateFont();
	}

	public void setFontName(String fontName) {

		this.fontName = fontName;
		updateFont();
	}

	public void setFontStyle(int fontStyle) {

		this.fontStyle = fontStyle;
		updateFont();
	}

	private void updateFont() {

		Font font 	= new Font(fontName, fontStyle, fframe.monitor.convertFloatToRealHeight(fontSize));	
		super.setFont(font);	
		repaint();		
	}
	
	public void setFontColor(Color fontColor) {
		this.fontColor = fontColor;
		
		fontColorEnabled = fontColor;
	}

	public Color getFontColor() {
		return fontColorEnabled;
	}
	
	public void setBackgroundColor(Color c) {
		
		this.backgroundColor = c;
		backgroundColorEnabled 	= c;
	}
	
	public void setBorderColor(Color c) {
		
		this.frameColor = c;
	}
	
	public void setBorderRadius(float radius) {

		this.radius = radius;
	}

	public void setBounds(float x, float y, float width, float height) {

		this.x 		= x;
		this.y 		= y;
		this.width 	= width;
		this.height 	= height;	
		updateBounds();
	}

	public void setLocation(float x, float y) {

		this.x 		= x;
		this.y 		= y;		
		updateBounds();
	}

	public void setSize(float width, float height) {

		this.width 	= width;
		this.height 	= height;	
		updateBounds();
	}

	private void updateBounds() {

		if(image != null) {
			
			resizeImage();
		}
		
		super.setBounds(fframe.monitor.convertFloatToRealWidth(x), fframe.monitor.convertFloatToRealHeight(y), fframe.monitor.convertFloatToRealWidth(width), fframe.monitor.convertFloatToRealHeight(height));
	}

	private void resizeImage() {
		
		imageResized = ImageTools.getScaledInstance(image, fframe.monitor.convertFloatToRealWidth(width), fframe.monitor.convertFloatToRealHeight(height), true);		
	}	

	
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
				
		if(enabled == false) {
			
			backgroundColor 	= backgroundColorDisabled;
			fontColor 		= fontColorDisabled;
		}
		else {
			
			backgroundColor 	= backgroundColorEnabled;
			fontColor 		= fontColorEnabled;
		}
		
		repaint();
	}		
	
	
	@Override 
	protected void paintComponent(Graphics g) {

		super.paintComponent(g);
		
		if (!isOpaque()) {

			int w = getWidth();
			int h = getHeight();					

			Graphics2D g2 = (Graphics2D) g.create();

			
			int r = fframe.monitor.convertFloatToRealHeight(radius);
			g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);

			
			borderSpaceWidth 	= fframe.monitor.convertFloatToRealWidth(0.00078125f);
			borderSpaceheight 	= fframe.monitor.convertFloatToRealHeight(0.0013888889f);
			
			
			
			if(image != null) {

				RoundRectangle2D round = new RoundRectangle2D.Float(); 
				round.setRoundRect(borderSpaceWidth, borderSpaceheight, w - borderSpaceWidth*2, h - borderSpaceheight*2, r, r);
				g2.setClip(round);

				g2.setRenderingHint(RenderingHints.KEY_ALPHA_INTERPOLATION, RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY);
				g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
				g2.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
				
				
				
				if(pressed && this.isEnabled()) 	{
					
					Color c = new Color(0, 0, 0, 30);
					g2.setColor(c);
					g2.fillRoundRect(borderSpaceWidth, borderSpaceheight, w - borderSpaceWidth*2, h - borderSpaceheight*2, r, r);
				}			
				else {
					
					g2.setColor(backgroundColor);
					g2.fillRoundRect(borderSpaceWidth, borderSpaceheight, w - borderSpaceWidth*2, h - borderSpaceheight*2, r, r);
				}
				
				g2.drawImage(imageResized, null,  0, 0);
				
				
				g2.setClip(null);
			}
			else {

				if(pressed && this.isEnabled()) 	g2.setColor(UIManager.getColor("Button.background"));
				else			g2.setColor(backgroundColor);

				g2.fillRoundRect(borderSpaceWidth, borderSpaceheight, w - borderSpaceWidth*2, h - borderSpaceheight*2, r, r);
			}
			
			
			g2.setColor(frameColor);
			g2.drawRoundRect(borderSpaceWidth, borderSpaceheight, w - borderSpaceWidth*2, h - borderSpaceheight*2, r, r);
			
			
			FontMetrics fm = g2.getFontMetrics();
			
			int stringWidth = fm.stringWidth(text)-1;
			
			LineMetrics metrics 	= fm.getLineMetrics(text, g);			
			int ascent 			= (int) metrics.getAscent();
			int descent 			= (int) metrics.getDescent();
			int fh 				= (int) metrics.getHeight();			
			int leading 			= fm.getLeading();
						
			
			//g2.drawLine(0, h/2, w, h/2);
			//g2.drawLine(0, h/2-ascent, w, h/2-ascent);
			//g2.drawLine(0, h/2+descent, w, h/2+descent);
			
			//g2.drawRect(w/2-stringWidth/2, (int)(h/2-metrics.getHeight()/2), stringWidth, (int)(metrics.getHeight()));
			
						
			g2.setFont(getFont());
			g2.setColor(fontColor);
			g2.drawString(text, w/2-stringWidth/2, h/2 - 1 + ascent/2);
			

			g2.dispose();
		}
	}

	@Override
	public void mouseClicked(MouseEvent e) {

	}

	@Override
	public void mousePressed(MouseEvent e) {		
		pressed = true;
		repaint();
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		pressed = false;
		repaint();
	}

	@Override
	public void mouseEntered(MouseEvent e) {

	}

	@Override
	public void mouseExited(MouseEvent e) {

	}
}
