package ffilechooser;

import java.awt.AWTEvent;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.awt.event.AWTEventListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JTextField;
import javax.swing.UIManager;

import ffilechooser.FMonitor.MonitorListener;

public class FHSlider extends JComponent implements MouseListener, MouseMotionListener {


	static public interface ChangedValueListener {

		void valueChanged(int arrowIndex, float value);
	}

	ChangedValueListener changedValueListener;	


	float x 				= 0;
	float y 				= 0;
	float width 			= 0.02f;
	float height 		= 0.01f;

	String fontName 		= "Arial";
	int fontStyle 		= Font.PLAIN;
	float fontSize 		= 0.01f;
	float radius 		= 0.005f;


	Color lineColor 		= Color.white;
	float arrowHalfSize 	= 0.0045f;
	int lineLength 		= 1;
	int startX 			= 0;
	int endX 			= 0;
	int lineY 			= 0;


	int grabbedArrowIndex 	= 0;
	int grabDiffXPos 	= 0;


	class Arrow {

		int x;
		int y;

		float range;
		float value;
		float minValue;
		float maxValue;
		Color textColor;
		Color arrowColor;
		Color borderArrowColor;

		public Arrow(int range, int minValue, int maxValue, int initValue, Color arrowColor, Color borderArrowColor, Color textColor) {

			this.range				= range;
			this.minValue 			= minValue;
			this.maxValue 			= maxValue;
			this.value 				= initValue;
			this.arrowColor 			= arrowColor;
			this.borderArrowColor 	= borderArrowColor;
			this.textColor 			= textColor;
		}

		public void setGrabLocation(int x, int y) {
			this.x = x;
			this.y = y;
		}
	}

	Vector<Arrow> arrows = new Vector<Arrow>();


	float monitorOldWidth = 0;
	float monitorOldHeight = 0;


	FFrame fframe;

	public FHSlider(final FFrame fframe, ChangedValueListener changedValueListener) {

		this.fframe 				= fframe;
		this.changedValueListener 	= changedValueListener;

		setBackground(new Color(0x0, true)); //Nimbus
		setBorder(BorderFactory.createEmptyBorder(4, 8, 4, 8));


		addMouseListener(this);
		addMouseMotionListener(this);


		setCursor(Cursor.getDefaultCursor());


		Toolkit.getDefaultToolkit().addAWTEventListener(new AWTEventListener() {

			@Override
			public void eventDispatched(AWTEvent event) {

				if(fframe.monitor.width != monitorOldWidth || fframe.monitor.height != monitorOldHeight) {

					monitorOldWidth = fframe.monitor.width;
					monitorOldHeight = fframe.monitor.height;
					updateBounds();
					updateFont();				
				}
			}
		}, AWTEvent.PAINT_EVENT_MASK);
	}



	public void addArrow(int range, int minValue, int maxValue, int initValue, Color arrowColor, Color borderArrowColor, Color textColor) {

		arrows.add(new Arrow(range, minValue,maxValue, initValue, arrowColor, borderArrowColor, textColor));
	}


	public void setValue(int arrowIndex, float value) {

		if(arrowIndex < 0 || arrowIndex > arrows.size()-1) return;

		arrows.elementAt(arrowIndex).value = value;
		repaint();	
	}


	public void setLineColor(Color c) {
		this.lineColor = c;
	}

	public void setFont(Font font) {				
		super.setFont(font);

		if(fframe == null) return;

		fontName 	= font.getName();
		fontStyle 	= font.getStyle();
		fontSize 	= fframe.monitor.convertRealToFloatHeight(font.getSize());
	}	

	public void setFontSize(float size) {

		this.fontSize = size;
		updateFont();
	}

	public void setFontName(String fontName) {

		this.fontName = fontName;
		updateFont();
	}

	public void setFontStyle(int fontStyle) {

		this.fontStyle = fontStyle;
		updateFont();
	}

	private void updateFont() {

		Font font 	= new Font(fontName, fontStyle, fframe.monitor.convertFloatToRealHeight(fontSize));	
		super.setFont(font);	

		updateBounds();
		repaint();		
	}


	public void setRadius(float radius) {

		this.radius = radius;
	}

	public void setBounds(float x, float y, float width, float height) {

		this.x 		= x;
		this.y 		= y;
		this.width 	= width;
		this.height 	= height;	
		updateBounds();
	}

	public void setLocation(float x, float y) {

		this.x 		= x;
		this.y 		= y;		
		updateBounds();
	}

	public void setSize(float width, float height) {

		this.width 	= width;
		this.height 	= height;	
		updateBounds();
	}

	private void updateBounds() {

		int h = fframe.monitor.convertFloatToRealHeight(height);

		if(getFont() != null) {
			FontMetrics metrics = getFontMetrics(getFont());
			h += metrics.getHeight();
		}

		setBounds(fframe.monitor.convertFloatToRealWidth(x), fframe.monitor.convertFloatToRealHeight(y), fframe.monitor.convertFloatToRealWidth(width), h);
	}

	private void checkArrowGrab(int x, int y) {

		if(grabbedArrowIndex != -1) return;

		int realArrowHalfSize 	= fframe.monitor.convertFloatToRealWidth(arrowHalfSize);

		for(int i=0;i<arrows.size();i++) {

			if((x > arrows.elementAt(i).x - realArrowHalfSize) && x < (arrows.elementAt(i).x + realArrowHalfSize) 
					&& (y > arrows.elementAt(i).y - realArrowHalfSize) && y < (arrows.elementAt(i).y + realArrowHalfSize/2) ) {

				grabbedArrowIndex = i;
				grabDiffXPos = arrows.elementAt(i).x-x;
				break;
			}			
		}
	}

	private void arrowDrag(int x, int y) {

		if(grabbedArrowIndex != -1) {

			Arrow grabbedArrow = arrows.elementAt(grabbedArrowIndex);

			float value =  (int) (((float)grabbedArrow.range/lineLength) * ((x+grabDiffXPos)-startX));
			if(value < 0) 						value = 0; 
			if(value < grabbedArrow.minValue)  	value = grabbedArrow.minValue;
			if(value > grabbedArrow.range) 		value = grabbedArrow.range; 
			if(value > grabbedArrow.maxValue)  	value = grabbedArrow.maxValue;

			grabbedArrow.value = value;

			changedValueListener.valueChanged(grabbedArrowIndex, arrows.elementAt(grabbedArrowIndex).value);
			repaint();
		}
	}

	private void arrowReleased() {

		grabbedArrowIndex = -1;
	}

	@Override 
	protected void paintComponent(Graphics g) {

		if (!isOpaque()) {		


			Graphics2D g2 = (Graphics2D) g.create();
			g2.setRenderingHint(RenderingHints.KEY_ALPHA_INTERPOLATION, RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY);
			g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
			g2.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);	



			float strokeSize = fframe.monitor.convertFloatToRealHeight(0.001f);
			if(strokeSize < 1) strokeSize = 1;

			g2.setStroke(new BasicStroke(strokeSize));		

			g2.setFont(getFont());			
			FontMetrics metrics 		= g2.getFontMetrics();
			int textHeight 			= g2.getFontMetrics().getHeight();

			int realArrowHalfSize 	= fframe.monitor.convertFloatToRealWidth(arrowHalfSize);
			lineLength 				= (getWidth()-realArrowHalfSize*2) -2;
			startX 					= realArrowHalfSize + 1;
			endX 					= (int) (startX+lineLength);
			lineY 					= (int) (textHeight+textHeight/3);


			g2.setColor(lineColor);			
			g2.drawLine(startX, lineY, endX, lineY);

			for(int i=0;i<arrows.size();i++) {

				Arrow arrow = arrows.elementAt(i);

				int xPos = startX + (int) (((float)lineLength/arrow.range) * arrow.value);	
				int yPos = lineY + realArrowHalfSize + 1;

				arrow.setGrabLocation(xPos, yPos);

				g2.setColor(arrow.arrowColor);
				g2.fillPolygon(new int[] {xPos-realArrowHalfSize, xPos+realArrowHalfSize, xPos}, new int[] {yPos, yPos, yPos-realArrowHalfSize}, 3);
				g2.setColor(arrow.borderArrowColor);
				g2.drawPolygon(new int[] {xPos-realArrowHalfSize, xPos+realArrowHalfSize, xPos}, new int[] {yPos, yPos, yPos-realArrowHalfSize}, 3);


				int strWidth = metrics.stringWidth("" + ((int)arrow.value));
				g2.setColor(arrow.textColor);			
				g2.drawString("" + (int)arrow.value, xPos - strWidth/2, textHeight);
			}

			g2.dispose();
		}
		super.paintComponent(g);
	}

	@Override
	public void mouseClicked(MouseEvent e) {

	}

	@Override
	public void mousePressed(MouseEvent e) {

		checkArrowGrab(e.getX(), e.getY());
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		arrowReleased();
	}

	@Override
	public void mouseEntered(MouseEvent e) {

	}

	@Override
	public void mouseExited(MouseEvent e) {

	}

	@Override
	public void mouseDragged(MouseEvent e) {

		arrowDrag(e.getX(), e.getY());
	}

	@Override
	public void mouseMoved(MouseEvent e) {


	}
}
