package ffilechooser;

import java.awt.AWTEvent;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Toolkit;
import java.awt.event.AWTEventListener;

import javax.swing.BorderFactory;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

public class FTextArea extends JScrollPane {

	float x 					= 0;
	float y 					= 0;
	float width 				= 0.02f;
	float height 			= 0.01f;


	Font basedFont 			= null;	
	String fontName 			= "Arial";
	int fontStyle 			= Font.PLAIN;
	float fontSize 			= 0.01f;

	Color backgroundColor 	= new Color(238,238,238);
	
	float monitorOldWidth 	= 0;
	float monitorOldHeight 	= 0;
	
	int borderSpaceWidth 	= 2;
	int borderSpaceheight 	= 2;
	
	float marginLeft 		= 0.0f;
	float marginTop 			= 0.0f;
	float marginRight 		= 0.0f;
	float marginBottom 		= 0.0f;
	
	FFrame fframe;	

	static JTextArea textArea = new JTextArea();
		
	public FTextArea(final FFrame fframe) {
		super(textArea);
		
		this.fframe = fframe;
		
		setBackground(backgroundColor); 
	
		setBorder(BorderFactory.createEmptyBorder(0,0,0,0));
		textArea.setBorder(BorderFactory.createEmptyBorder(0,0,0,0));
		
		
		Toolkit.getDefaultToolkit().addAWTEventListener(new AWTEventListener() {

			@Override
			public void eventDispatched(AWTEvent event) {

				if(fframe.monitor.width != monitorOldWidth || fframe.monitor.height != monitorOldHeight) {

					monitorOldWidth = fframe.monitor.width;
					monitorOldHeight = fframe.monitor.height;
					updateBounds();
					repaint();		
				}
			}
		}, AWTEvent.PAINT_EVENT_MASK);
	}
	
	public void setText(String s) {
		textArea.setText(s);		
	}	
	
	public void setMargin(float left, float top, float right, float bottom) {
		marginLeft 		= left;
		marginTop 		= top;
		marginRight 		= right;
		marginBottom 	= bottom;
	}

	public void setFont(Font basedFont, Font font) {			
		textArea.setFont(font);
		
		if(fframe == null) return;

		fontName 	= font.getName();
		fontStyle 	= font.getStyle();
		fontSize 	= fframe.monitor.convertRealToFloatHeight(font.getSize());	
				
		if(basedFont != null) this.basedFont = basedFont;			
		updateFont();
	}	
	
	public void setFont(Font font) {				
		textArea.setFont(font);
		
		if(fframe == null) return;
		
		fontName 	= font.getName();
		fontStyle 	= font.getStyle();
		fontSize 	= fframe.monitor.convertRealToFloatHeight(font.getSize());		
				
		updateFont();
	}	
	
	public void setFontSize(float size) {
		
		this.fontSize = size;
		updateFont();
	}
	
	public void setFontName(String fontName) {
		
		this.fontName = fontName;
		updateFont();
	}
	
	public void setFontStyle(int fontStyle) {
		
		this.fontStyle = fontStyle;
		updateFont();
	}
	
	public void setFontColor(Color color) {		
		textArea.setForeground(color);
	}	
	
	private void updateFont() {
						
		Font font 	= new Font(fontName, fontStyle, fframe.monitor.convertFloatToRealHeight(fontSize));		
		
		FontMetrics metrics 	= getFontMetrics(font);		
		int textHeight 		= metrics.getHeight();
		int textAsc 			= metrics.getDescent();
		
		//height 				= fframe.monitor.convertRealToFloatHeight(textHeight + textAsc);
		
		if(basedFont != null) {		
			font = basedFont.deriveFont(fontStyle, fframe.monitor.convertFloatToRealHeight(fontSize));
		}
		
		textArea.setFont(font);	
		repaint();		
	}
	
	
	public void setBackgroundColor(Color c) {		
		this.backgroundColor = c;
		
		setBackground(backgroundColor); 
		textArea.setBackground(backgroundColor); 
	}
	
	public void setBounds(float x, float y, float width, float height) {
		
		this.x 		= x;
		this.y 		= y;
		this.width 	= width;				
		this.height 	= height;				
		updateBounds();
	}

	public void setBounds(int x, int y, int width, int height) {
		super.setBounds(x, y, width, height);
		
		this.x 		= fframe.monitor.convertRealToFloatWidth(x);
		this.y 		= fframe.monitor.convertRealToFloatHeight(y);
		this.width 	= fframe.monitor.convertRealToFloatWidth(width);
		this.height 	= fframe.monitor.convertRealToFloatHeight(height);
		updateBounds();
	}
	
	public void setLocation(float x, float y) {

		this.x 		= x;
		this.y 		= y;		
		updateBounds();
	}

	public void setSize(float width, float height) {

		this.width 	= width;
		this.height 	= height;	
		updateBounds();
	}
	
	public void setEditable(boolean editable) {
		textArea.setEditable(editable);
	}
	
	private void updateBounds() {
				
		updateFont();
		super.setBounds(
				fframe.monitor.convertFloatToRealWidth(x+marginLeft), 
				fframe.monitor.convertFloatToRealHeight(y+marginTop), 
				fframe.monitor.convertFloatToRealWidth(width-(marginLeft + marginRight)), 
				fframe.monitor.convertFloatToRealHeight(height-(marginBottom)));	
	}

}
