package ffilechooser;

import java.awt.AWTEvent;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.awt.event.AWTEventListener;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JTextField;
import javax.swing.UIManager;

import ffilechooser.FMonitor.MonitorListener;

public class FTextField extends JTextField {

	float x 			= 0;
	float y 			= 0;
	float width 		= 0.02f;
	float height 	= 0.01f;

	String fontName 	= "Arial";
	int fontStyle 	= Font.PLAIN;
	float fontSize 	= 0.01f;
	float radius 	= 0.005f;
	
	Color borderColor = Color.GRAY;
	
	

	float monitorOldWidth = 0;
	float monitorOldHeight = 0;
	

	int borderSpaceWidth = 2;
	int borderSpaceheight = 2;
	
	
	FFrame fframe;	
	
	public FTextField(final FFrame fframe) {

		this.fframe = fframe;
		
		setBackground(new Color(0x0, true)); //Nimbus
		setBorder(BorderFactory.createEmptyBorder(4, 10, 4, 8));
		
		
		Toolkit.getDefaultToolkit().addAWTEventListener(new AWTEventListener() {

			@Override
			public void eventDispatched(AWTEvent event) {

				if(fframe.monitor.width != monitorOldWidth || fframe.monitor.height != monitorOldHeight) {

					monitorOldWidth = fframe.monitor.width;
					monitorOldHeight = fframe.monitor.height;
					updateBounds();
					repaint();		
				}
			}
		}, AWTEvent.PAINT_EVENT_MASK);
	}
	
	public void setText(String s) {
		super.setText(s);		
	}	

	public void setFont(Font font) {				
		super.setFont(font);
		
		if(fframe == null) return;
		
		fontName 	= font.getName();
		fontStyle 	= font.getStyle();
		fontSize 	= fframe.monitor.convertRealToFloatHeight(font.getSize());		
				
		updateFont();
	}	
	
	public void setFontSize(float size) {
		
		this.fontSize = size;
		updateFont();
	}
	
	public void setFontName(String fontName) {
		
		this.fontName = fontName;
		updateFont();
	}
	
	public void setFontStyle(int fontStyle) {
		
		this.fontStyle = fontStyle;
		updateFont();
	}
	
	private void updateFont() {
						
		Font font 	= new Font(fontName, fontStyle, fframe.monitor.convertFloatToRealHeight(fontSize));		
		
		FontMetrics metrics 	= getFontMetrics(font);		
		int textHeight 		= metrics.getHeight();
		int textAsc 			= metrics.getDescent();
		
		height 				= fframe.monitor.convertRealToFloatHeight(textHeight + textAsc);
				
		super.setFont(font);	
		repaint();		
	}
	
	
	public void setBorderRadius(float radius) {
		
		this.radius = radius;
	}
	
	public void setBorderColor(Color c) {
		this.borderColor = c;
	}
	
	public void setBounds(float x, float y, float width, float height) {
		
		this.x 		= x;
		this.y 		= y;
		this.width 	= width;				
		updateBounds();
	}

	public void setBounds(int x, int y, int width, int height) {
		super.setBounds(x, y, width, height);
		
		this.x 		= fframe.monitor.convertRealToFloatWidth(x);
		this.y 		= fframe.monitor.convertRealToFloatHeight(y);
		this.width 	= fframe.monitor.convertRealToFloatWidth(width);
		updateBounds();
	}
	
	public void setLocation(float x, float y) {

		this.x 		= x;
		this.y 		= y;		
		updateBounds();
	}

	public void setSize(float width, float height) {

		this.width 	= width;
		this.height 	= height;	
		updateBounds();
	}
	
	private void updateBounds() {
				
		updateFont();
		super.setBounds(fframe.monitor.convertFloatToRealWidth(x), fframe.monitor.convertFloatToRealHeight(y), fframe.monitor.convertFloatToRealWidth(width), fframe.monitor.convertFloatToRealHeight(height));
	}

	
	@Override 
	protected void paintComponent(Graphics g) {

		if (!isOpaque()) {

			int w = getWidth();
			int h = getHeight();					
			
			Graphics2D g2 = (Graphics2D) g.create();
			
			g2.setFont(getFont());
			
			borderSpaceWidth 	= fframe.monitor.convertFloatToRealWidth(0.00078125f);
			borderSpaceheight 	= fframe.monitor.convertFloatToRealHeight(0.0013888889f);
			
			
			int r = fframe.monitor.convertFloatToRealHeight(radius);
			g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
			g2.setColor(UIManager.getColor("TextField.background"));
			g2.fillRoundRect(borderSpaceWidth, borderSpaceheight, w - borderSpaceWidth*2, h - borderSpaceheight*2, r, r);
			g2.setColor(borderColor);
			g2.drawRoundRect(borderSpaceWidth, borderSpaceheight, w - borderSpaceWidth*2, h - borderSpaceheight*2, r, r);
			g2.dispose();
		}
		super.paintComponent(g);
	}
}
