package ffilechooser;


import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Transparency;
import java.awt.color.ColorSpace;
import java.awt.color.ICC_ColorSpace;
import java.awt.color.ICC_Profile;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.List;

import javax.imageio.ImageIO;
import javax.imageio.spi.IIORegistry;


public class ImageTools {


	static public String encodeColorToString(Color color) {

		String hex = String.format("#%02x%02x%02x", color.getRed(), color.getGreen(), color.getBlue());  
		return hex;

	}

	static public BufferedImage getScaledInstance(BufferedImage img, int targetWidth,	int targetHeight, boolean higherQuality) {

		int type = (img.getTransparency() == Transparency.OPAQUE) ? BufferedImage.TYPE_INT_RGB : BufferedImage.TYPE_INT_ARGB;

		BufferedImage ret = (BufferedImage)img;
		int w, h;

		if (higherQuality) {

			// Use multi-step technique: start with original size, then
			// scale down in multiple passes with drawImage()
			// until the target size is reached
			w = img.getWidth();
			h = img.getHeight();
		} 
		else {
			// Use one-step technique: scale directly from original
			// size to target size with a single drawImage() call
			w = targetWidth;
			h = targetHeight;
		}

		do {

			// * Weil immer durch 2 geteilt wird, dann pruefen ob durch 2 teilbar, wenn nicht, dann aufrunden *
			if (targetWidth % 2 != 0) targetWidth +=1;
			if (targetHeight % 2 != 0) targetHeight +=1;


			if (higherQuality && w > targetWidth) {
				w /= 2;
				if (w < targetWidth) {
					w = targetWidth;
				}
			}
			else w = targetWidth; // * Wenn Image schon kleiner, dann nicht mehr weiter, sondern Schleife am Ende mit Target-Groesse beenden *


			if (higherQuality && h > targetHeight) {
				h /= 2;
				if (h < targetHeight) {
					h = targetHeight;
				}
			}
			else h = targetHeight;  // * Wenn Image schon kleiner, dann nicht mehr weiter, sondern Schleife am Ende mit Target-Groesse beenden *

			BufferedImage tmp = new BufferedImage(w, h, type);
			Graphics2D g2 = tmp.createGraphics();
			if(higherQuality) {
				g2.setRenderingHint(RenderingHints.KEY_ALPHA_INTERPOLATION, RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY);
				//g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);				
				g2.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
			}
			g2.drawImage(ret, 0, 0, w, h, null);
			g2.dispose();

			ret = tmp;

		} 
		while (w != targetWidth || h != targetHeight);

		return ret;
	}




	static public int getRasterPos(long pos, float raster) {

		int newpos = (int) ((int)((float)pos/raster)*raster);
		return newpos;
	}   




	static public String getColorHexString(Color c)  {

		String cString = String.format("#%06x", c.getRGB() & 0xFFFFFF);	

		return cString;
	}

	static public float[] rgbToCmyk(int rgb, ColorSpace cmykColorspace) {

		float[] fromRGB = new float[4];

		try {
			
			float[] c = new float[3];
			c[0] = (1.0f/255)*((rgb >> 16) & 0xff);
			c[1] = (1.0f/255)*((rgb >> 8) & 0xff);
			c[2] = (1.0f/255)*((rgb) & 0xff);

			fromRGB = cmykColorspace.fromRGB(c);			
		}
		catch(Exception e) { e.printStackTrace();}

		return fromRGB;
	}


	static public int cmykToRgb(float[] cmyk, ColorSpace rgbColorspace) {

		int rgb = 0;

		try {
			float[] fRGB = rgbColorspace.toRGB(cmyk);			
			rgb =  (int)(fRGB[0]*255) << 16 | (int)(fRGB[1]*255) << 8 |  (int)(fRGB[2]*255);		
		}
		catch(Exception e) { e.printStackTrace();}

		return rgb;
	}


	static public float convertToLogarith(float biegung, float max, float value) {

		float result = 0;
		if(value >= 0) 	{

			float temp = (1.0f/max*value);
			result = (float) (temp * max * Math.pow(biegung, 1 - temp));								
		}
		else {

			float temp = (1.0f/max*(value*-1));
			result = (float) ((temp * max * Math.pow(biegung, 1 - temp))*-1);								
		}

		return result;
	}
}
