package main;
import java.awt.AWTEvent;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.event.AWTEventListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.prefs.Preferences;

import javax.imageio.ImageIO;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.SwingUtilities;
import javax.swing.border.MatteBorder;
import javax.xml.crypto.Data;

import d64.D64;
import d64.DirEntry;
import ffilechooser.Directory;
import ffilechooser.FFrame;
import ffilechooser.FPanel;

public class C64Dir extends FPanel {


	public boolean activeDirectory 		= false;

	String volumeName = "-1";
	String oldVolumeName = "";

	public C64DirectoryTable directoryTable;

	public String[] columnNames = {"0","", ""};

	public String directoryPath = "/Volumes";
	public String directoryPathOld = "/Volumes";


	Preferences prefs = Preferences.userNodeForPackage(C64Dir.class);
	String PREF_NAME_directory_path 	= "c64_directory_path";


	String mouseClickCheckLastClickedCellName = "";


	BufferedImage appIconImage 					= null;
	BufferedImage dirImage 						= null;
	BufferedImage fileImage 						= null;
	BufferedImage appImage 						= null;

	public BufferedImage backImage 				= null;
	BufferedImage backOffImage 					= null;	
	public BufferedImage forwardImage 			= null;
	public BufferedImage forwardOffImage 		= null;


	JLabel titleLabel;


	public int width = 1;

	boolean showHiddenFiles 		= false;

	C64Dir directory;

	public D64 d64;


	public int directoryIconRowWidth;


	// * Directory *

	public Font directoryTableHeaderFont;
	public int directoryTableHeaderRowHeight;
	public int directoryTableRowHeight;

	public int directoryNameRowMinWidth;
	public int directoryModifiedRowWidth;
	public int directorySizeRowWidth;
	public int directoryPermissionsRowWidth;
	public int directoryLastRowWidth;

	public Font directoryTableFont;
	public int directoryIconWidth;


	// * Drives *
	public float drivesPanelImageMultip  = 0.65f;

	public int drivesPanelCellWidth;
	public int drivesPanelCellHeight;


	float monitorOldWidth = 0;
	float monitorOldHeight = 0;


	long availableDiskSpace 	= 0;
	public boolean directoryLoading = false;


	public MacC64FileOperator macC64FileOperator;

	String[] favorites;	

	public int side = 0;

	public String selectedName = "";

	public C64Dir(MacC64FileOperator macC64FileOperator, int side, String[] favorites, String fontName) {

		super(macC64FileOperator.fframe);

		this.side = side;

		this.macC64FileOperator 	= macC64FileOperator;
		this.favorites 			= favorites;

		this.directory 			= this;

		this.directoryTableHeaderFontName 		= fontName;
		this.directoryTableFontName 				= fontName;

		directoryPath = prefs.get(PREF_NAME_directory_path, directoryPath);


		Toolkit.getDefaultToolkit().addAWTEventListener(new AWTEventListener() {

			@Override
			public void eventDispatched(AWTEvent event) {

				if(macC64FileOperator.fframe.monitor.width != monitorOldWidth || macC64FileOperator.fframe.monitor.height != monitorOldHeight) {

					monitorOldWidth = macC64FileOperator.fframe.monitor.width;
					monitorOldHeight = macC64FileOperator.fframe.monitor.height;

					updateUI();		

					if(directoryTable != null) {

						directoryTable.getTableHeader().	setBorder(new MatteBorder(0,	directory.macC64FileOperator.fframe.monitor.convertFloatToRealWidth(0.011f),0,0,directoryTable.backgroundColor));
						directoryTable.getTableHeader().setPreferredSize(new Dimension(getWidth(), directory.directoryTableHeaderRowHeight));
						directoryTable.setRowHeight( directory.directoryTableRowHeight );			
						directoryTable.getTableHeader().setFont(directory.directoryTableHeaderFont);					
						directoryTable.setDirectoryDesign();	
					}
				}
			}
		}, AWTEvent.PAINT_EVENT_MASK);

		updateUI();		



		// * Abfrage ob Verzeichnis beim Start existiert *
		// * Wenn ein Laufwerk entfernt wurde, und dann das Verzeichnis nicht mehr existiert, auf das /Volumes Verzeichnis wechseln *
		if(new File(directoryPath).exists() == false) directoryPath = "/Volumes";



		try	{ InputStream in = getClass().getResourceAsStream("/ffileshooser_pack/dir.png"); dirImage = ImageIO.read(in);}
		catch( Exception e ) {e.printStackTrace();}

		try	{ InputStream in = getClass().getResourceAsStream("/ffileshooser_pack/file.png"); fileImage = ImageIO.read(in);}
		catch( Exception e ) {e.printStackTrace();}

		try	{ InputStream in = getClass().getResourceAsStream("/ffileshooser_pack/back.png"); backImage = ImageIO.read(in);}
		catch( Exception e ) {e.printStackTrace();}

		try	{ InputStream in = getClass().getResourceAsStream("/ffileshooser_pack/back_off.png"); backOffImage = ImageIO.read(in);}
		catch( Exception e ) {e.printStackTrace();}

		try	{ InputStream in = getClass().getResourceAsStream("/ffileshooser_pack/forward.png"); forwardImage = ImageIO.read(in);}
		catch( Exception e ) {e.printStackTrace();}

		try	{ InputStream in = getClass().getResourceAsStream("/ffileshooser_pack/forward_off.png"); forwardOffImage = ImageIO.read(in);}
		catch( Exception e ) {e.printStackTrace();}

		try	{ InputStream in = getClass().getResourceAsStream("/ffileshooser_pack/app.png"); appImage = ImageIO.read(in);}
		catch( Exception e ) {e.printStackTrace();}


		directoryTable 	= new C64DirectoryTable(this);	

		directoryTable.setBackground(directoryTable.backgroundColor);

		updateUI();
		updateViews();


		addComponentListener(new ComponentListener(){

			@Override
			public void componentHidden(ComponentEvent arg0) {}
			@Override
			public void componentMoved(ComponentEvent e) {}
			@Override
			public void componentShown(ComponentEvent arg0) {}
			@Override
			public void componentResized(ComponentEvent e) {

				updateUI();

				// * AutoResize *
				updateViews();

				revalidate();
				
				directoryTable.fillTableWithEmptyRows(); 
			}
		});


		SwingUtilities.invokeLater(new Runnable() {
			public void run() {

				directoryTable.setDirectoryDesign();	
				directoryTable.invalidate();

				showDirectory();
			}
		});
	}

	public void setHeaderTitle(String name, String id) {

		if(name.length() < 16) {		

			int spaces = 16-name.length();
			for(int i=0;i<spaces;i++) name += " ";
		}

		columnNames[1] = "\""+ name +"\"";
	}



	public String directoryTableHeaderFontName 		= "";
	public int directoryTableHeaderFontType 			= Font.PLAIN;
	public float directoryTableHeaderFontSize 		= 0.013f;

	public String directoryTableFontName 			= "";
	public int directoryTableFontType 				= Font.PLAIN;
	public float directoryTableFontSize 				= 0.013f;



	public void updateUI() {

		directoryIconRowWidth 			= macC64FileOperator.fframe.monitor.convertFloatToRealWidth(0.02471875f);
		directoryTableHeaderFont 		= new Font(directoryTableHeaderFontName, directoryTableHeaderFontType, macC64FileOperator.fframe.monitor.convertFloatToRealHeight(directoryTableHeaderFontSize));
		directoryTableHeaderRowHeight 	= macC64FileOperator.fframe.monitor.convertFloatToRealHeight(0.011361112f);
		directoryTableFont				= new Font(directoryTableFontName, directoryTableFontType, macC64FileOperator.fframe.monitor.convertFloatToRealHeight(directoryTableFontSize));
		directoryTableRowHeight 			= macC64FileOperator.fframe.monitor.convertFloatToRealHeight(0.011361112f);
		directoryNameRowMinWidth 		= macC64FileOperator.fframe.monitor.convertFloatToRealWidth(0.11f);
		directoryModifiedRowWidth 		= macC64FileOperator.fframe.monitor.convertFloatToRealWidth(0.05859375f);
		directorySizeRowWidth 			= macC64FileOperator.fframe.monitor.convertFloatToRealWidth(0.05096875);
		directoryPermissionsRowWidth 	= macC64FileOperator.fframe.monitor.convertFloatToRealWidth(0.03078125f);
		directoryLastRowWidth 			= macC64FileOperator.fframe.monitor.convertFloatToRealWidth(0.0078125f);
		directoryIconWidth 				= macC64FileOperator.fframe.monitor.convertFloatToRealWidth(0.00625f);

		drivesPanelCellWidth 			= macC64FileOperator.fframe.monitor.convertFloatToRealWidth(0.06875f);
		drivesPanelCellHeight 			= macC64FileOperator.fframe.monitor.convertFloatToRealHeight(0.018055556f);
	}


	private void updateViews() {

		// * Directory Inhalt updaten *
		directoryTable.setBounds(0, 0, getWidth(), getHeight());
		directoryTable.getScrollPane().setBounds(0, 0, getWidth(), getHeight());	
	}

	public void clearSelection() {

		directoryTable.clearSelection();
	}


	public void switchAllToolsOff() {

		directoryTable.setVisible(false); 
		directoryTable.getScrollPane().setVisible(false); 
		
		remove(directoryTable.getScrollPane());
	}


	public void showDirectory() {

		if(getComponentZOrder(directoryTable.getScrollPane()) == -1) add(directoryTable.getScrollPane());
		directoryTable.setVisible(true);		
		directoryTable.getScrollPane().setVisible(true);		
		reloadDir();
	}


	public void reloadDir() {

		SwingUtilities.invokeLater(new Runnable() {
			public void run() {

				int[] rows = directoryTable.getSelectedRows();
				loadDirectory(directoryPath, false);


				directoryTable.invalidate();				
			}
		});	
	}

	public void clickedDirectoryColumn(String name) {

		setSelectedFile(name);		
	}

	public void doubleClickedDirectoryColumn(String name) {

		setSelectedFile(name);
	}

	private void setSelectedFile(String name) {

		selectedName = name;

		String newPath = "";

		if(new File(name).isDirectory()) newPath = name;
		else {

			newPath = directoryPath + "/" +  name;
			newPath = newPath.replace("//", "/");				
		}	
	}


	public void updateSelectedBlocksLabel(int[] selectedRows) {

		int blocks = 0;

		// * Selected ALL *
		if(selectedRows != null) {

			try {

				for(int i=0;i<selectedRows.length;i++) blocks += d64.d64FileList.get(selectedRows[i]).getFileBlockSize();
			}
			catch(Exception e) {	}
		}
		else {

			// * Selected a few files *

			String[] selFiles = getSelectedFiles();
			if(selFiles != null) {
				for(int i=0;i<selFiles.length;i++) {
					for(int d=0;d<d64.d64FileList.size();d++) {

						if(d64.d64FileList.get(d).getFileName().compareTo(selFiles[i]) == 0) blocks += d64.d64FileList.get(d).getFileBlockSize();
					}
				}
			}
		}


		if(side == 0)  	macC64FileOperator.gui.dirLeftSelectedBlocks.setText("["+blocks + " Blocks selected]");
		else 			macC64FileOperator.gui.dirRightSelectedBlocks.setText("[" + blocks + " Blocks selected]");
		
		macC64FileOperator.gui.updateButtons();
	}


	public String[] getSelectedFiles() {

		int[] selectedRows 		= directoryTable.getSelectedRows();		
		String[] selectedFiles 	= new String[selectedRows.length];

		try {
			for(int i=0;i<selectedRows.length;i++) selectedFiles[i] = directoryTable.getValueAt(selectedRows[i], 1).toString();
		}
		catch(Exception e) {
			return null;
		}

		return selectedFiles;
	}


	public boolean waitMode = false;

	public void animateWaitCursor() {

		Thread T=new Thread(new Runnable() {

			@Override
			public void run() {

				try {

					waitMode = true;

					Thread.sleep(500); // *  erst nach 500ms anzeigen, weil sonst fast immer der Ball angezeigt wird! *

					while(waitMode) {

						for(int i=0;i<32;i++) {

							if(waitMode == false) break;
							setCursor( Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR) );  

							try {Thread.sleep(25);} catch (InterruptedException e) {e.printStackTrace();	}
						}
					}
					setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
				}
				catch(Exception e) {e.printStackTrace();}
			}
		} );
		T.start();		

	}

	public void loadDirectory(String path, boolean addToMemory) {

		directoryLoading = true;

		try {				

			setSelectedFile(path);				

			d64 = new D64(this, path);


			// * Disk Name *
			columnNames[1] = "\"" + d64.diskName;
			if(columnNames[1].length() < 16) {

				int count = columnNames[1].length();
				for(int i=0;i<=16-count;i++) columnNames[1] += " ";
			}
			columnNames[1] += "\"";		

			// * Disk ID *
			columnNames[2] = d64.diskID;

			// * Disk File Name List *
			Object[][] liste = new Object[d64.d64FileList.size()+1][3];
			for(int i=0;i<d64.d64FileList.size();i++) {

				liste[i][0] = "" + d64.d64FileList.get(i).getFileBlockSize();
				liste[i][1] = d64.d64FileList.get(i).getFileName();
				liste[i][2] = d64.d64FileList.get(i).getFileTypeString();
			}

			// * Add: "BLOCKS FREE." to the Table *
			liste[d64.d64FileList.size()][0] = "" + d64.freeBlocks();
			liste[d64.d64FileList.size()][1] = "BLOCKS FREE.";
			liste[d64.d64FileList.size()][2] = "";


			directoryTable.tableModel.setDataVector(liste, columnNames);	
			directoryTable.invalidate();

			directoryPath = path; // * Wenn Directory geoeffnet werden konnte, neuen Pfad uebernehmen * 
			prefs.put(PREF_NAME_directory_path, directoryPath);


			String name = new File(directoryPath).getName();
			if(name.compareTo("") == 0) name = "/";


			// * Rows fuellen *

			if(directoryTable.getHeight() > 0 && directoryTable.getScrollPane().getViewport().getHeight() > 0 && directoryTable.getHeight() < (directory.getHeight())) {

				directoryTable.fillTableWithEmptyRows();		
			}
			else {

				// * Solange warten bis die Scrollpane mit der Tabelle aufgebaut ist, dann erst die leeren Zeilen fuellen *
				Thread T=new Thread(new Runnable() {
					@Override
					public void run() {
						try {	

							// * Warten bis sich JTable in der vollen Hoehe aufgebaut hat... und dann erst mit Leerzeilen füllen *

							int count = 0;
							while(directory.getHeight() == 0 || directoryTable.getScrollPane().getViewport().getHeight() == 0 || directoryTable.getHeight() < (directory.getHeight())) {

								Thread.sleep(50);

								count++;
								if(count > 40) break;								
							};

							directoryTable.fillTableWithEmptyRows();						
						}
						catch(Exception e) {e.printStackTrace();}

						directoryLoading = false;
					}
				} );
				T.start();		
			}

			if(side == 0)  	macC64FileOperator.gui.dirLeftSelectedBlocks.setText("[0 Blocks selected]");
			else 			macC64FileOperator.gui.dirRightSelectedBlocks.setText("[0 Blocks selected]");
			
			updateViews();

		}
		catch(Exception e) {
			directoryLoading = false;
		}		

		directoryTable.setDirectoryDesign();		
	}

	public void activateDirectory() {

		activeDirectory = true;		
		directoryTable.getTableHeader().setBackground(new Color(0xb8eebe));
	}

	public void deactivateDirectory() {

		activeDirectory = false;		
		directoryTable.getTableHeader().setBackground(new Color(0xf0f0f0));
	}

}