package main;



import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.MouseInfo;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Vector;
import java.util.prefs.Preferences;

import javax.swing.AbstractAction;
import javax.swing.AbstractCellEditor;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.RowSorter;
import javax.swing.SortOrder;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.border.LineBorder;
import javax.swing.border.MatteBorder;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.plaf.ColorUIResource;
import javax.swing.plaf.basic.BasicScrollBarUI;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;
import javax.swing.text.BadLocationException;

import ffilechooser.Functions;
import ffilechooser.JImage;


public class C64DirectoryTable extends JTable  {

	Color foregroundColor 	= new Color(0xaea5fd);
	Color backgroundColor 	= new Color(0x4e44d9);
	Color scrollColor 		= new Color(0x867dc4);

	private JScrollPane accountListScrollPane;

	DefaultTableModel tableModel;

	C64Dir directory;
	C64DirectoryTable directoryTable;


	long doubleClicktime 		= System.currentTimeMillis();
	int doubleClickCount 		= 0;
	boolean doubleClick 			= false;
	int doubleClickGrenze 		= 400;
	int oldRow 					= -1;



	public void changeSelection(int rowIndex, int columnIndex, boolean toggle, boolean extend) {

		boolean selected = false;
		int[] rows = this.getSelectedRows();
		for(int i=0;i<rows.length;i++) if(rowIndex == rows[i])  selected = true;


		// * Wenn diese Row noch nicht selektiert, dann Selektion zulassen * 				
		super.changeSelection(rowIndex, columnIndex, toggle, extend);		
	}


	
	long titleLastClickTime = System.currentTimeMillis();
	int titleClickCount = 0;

	public C64DirectoryTable(final C64Dir directory) {

		this.directory = directory;
		this.directoryTable = this;


		tableModel = new DefaultTableModel() {

			@Override
			public boolean isCellEditable(int row, int column) {

				return false;
			}
		};
		setModel(tableModel);



		setAutoCreateRowSorter(false);
		addFocusListener(new FocusListener() {
			@Override
			public void focusGained(FocusEvent arg0) {	

			}
			@Override
			public void focusLost(FocusEvent arg0) {	

			}			
		});
		setFillsViewportHeight( true );


		setShowGrid(false);
		setShowVerticalLines(false);
		setIntercellSpacing( new Dimension(0, 0) );       

		// * Verhindert im Row Edit-Mode, dass nach ENTER die naechste Zeile aktiviert wird *
		getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0, false), "selectNextColumnCell");



		// * Row Height *
		setRowHeight( directory.directoryTableRowHeight );

		getTableHeader().setReorderingAllowed( false );
		getTableHeader().setResizingAllowed( true );			

		getTableHeader().setForeground(backgroundColor);
		getTableHeader().setBackground(foregroundColor);



		// * Header Height *
		getTableHeader().setPreferredSize(new Dimension(getWidth(), directory.directoryTableHeaderRowHeight));

		// * Header *
		TableCellRenderer baseRenderer = getTableHeader().getDefaultRenderer();
		getTableHeader().setBorder(new MatteBorder(0,directory.macC64FileOperator.fframe.monitor.convertFloatToRealWidth(0.011f),0,0,backgroundColor));
		getTableHeader().setDefaultRenderer(new TableHeaderRenderer(baseRenderer));
		getTableHeader().setFont(directory.directoryTableHeaderFont);
		getTableHeader().setEnabled(false); // !!!!!! Sehr wichtig, das verhindert die Standard Header Events (doppelte Sortierungen!)
		getTableHeader().addMouseListener(new MouseListener() {

			@Override
			public void mouseReleased(MouseEvent e) {}

			@Override
			public void mouseClicked(MouseEvent e) {	}

			@Override
			public void mousePressed(MouseEvent e) {	

				if(e.getButton() != 1) return;
				
				if(System.currentTimeMillis()-titleLastClickTime > 500) titleClickCount = 0;
				titleLastClickTime = System.currentTimeMillis();
						
				titleClickCount++;
				if(titleClickCount == 2) directory.macC64FileOperator.renameFile(directory.side, true);				
			}

			@Override
			public void mouseEntered(MouseEvent e) {	}

			@Override
			public void mouseExited(MouseEvent e) {	}
		});

		setFont(directory.directoryTableFont);




		// **** MAIN Listeners ****


		addFocusListener(new FocusListener() {

			@Override
			public void focusGained(FocusEvent arg0) {		

				directory.activateDirectory();
			}

			@Override
			public void focusLost(FocusEvent arg0) {	
			}			
		});

		addMouseMotionListener(new MouseMotionListener() {

			@Override
			public void mouseDragged(MouseEvent e) {				
			}

			@Override
			public void mouseMoved(MouseEvent e) {	
			}
		});

		addMouseListener(new MouseListener() {

			@Override
			public void mouseClicked(MouseEvent event) {}

			@Override
			public void mouseEntered(MouseEvent arg0) {}

			@Override
			public void mouseExited(MouseEvent arg0) {}

			@Override
			public void mousePressed(MouseEvent event) {	}

			@Override
			public void mouseReleased(MouseEvent event) {
				
				directory.updateSelectedBlocksLabel(null);	// To show "0 Blocks" (all deselected ... Mouse clicked into no rows) 
							
				if(event.getButton() != 1) return;
				

				final int row 		= rowAtPoint(event.getPoint());
				final int column 	= columnAtPoint(event.getPoint());

				boolean clickNameHasChanged = false;				
				if(row == -1 || column == -1 || getValueAt(row, 1) == null) {

					clickNameHasChanged = true;
					return;
				}


				// * Directory Double Click Routine *
				if(clickNameHasChanged) {

					doubleClick 	= false;
					doubleClickCount = 1;
					doubleClicktime = System.currentTimeMillis();	

					// * Wenn eine Zeile angeklickt wurde, und keine Sondertasten gedrueckt um Mehrfachselektion zu starten, dann alle anderen selektierten Zeilen deselektieren  *
					if(event.isShiftDown() == false && event.isMetaDown() == false &&  event.isAltDown() == false) directory.directoryTable.setRowSelectionInterval(row, row);	
				}
				else {

					if(System.currentTimeMillis() - doubleClicktime >= doubleClickGrenze || oldRow != row) {

						doubleClick 	= false;
						doubleClickCount = 1;
						doubleClicktime = System.currentTimeMillis();
						oldRow = row;
					}
					else {

						doubleClickCount++;						
						if(doubleClickCount >= 2) doubleClick = true;
						doubleClicktime = System.currentTimeMillis();
					}
				}



				// * Directory Laden / oder APP starten, sowie "Wait Mouse Cursor" anzeigen! *

				if(doubleClick) {

					doubleClickCount 	= 0;

					if(row != -1 && column != -1) directory.doubleClickedDirectoryColumn(((String)getValueAt(row, 1)).replace("\"",""));					
				}	
				else {

					if(row != -1 || column != -1) directory.clickedDirectoryColumn(((String)getValueAt(row, 1)).replace("\"",""));						
				}
								
			}
		});	



		// * Mouse Wheel Speed Scroll Korrektur *
		addMouseWheelListener(new MouseWheelListener() {

			@Override
			public void mouseWheelMoved(MouseWheelEvent e) {

				accountListScrollPane.getVerticalScrollBar().setUnitIncrement(5*Math.abs(e.getWheelRotation()));
				accountListScrollPane.dispatchEvent(e);				        
			}
		});



		addKeyListener(new KeyListener() {

			@Override
			public void keyTyped(KeyEvent e) {}

			@Override
			public void keyPressed(KeyEvent event) {	}

			@Override
			public void keyReleased(KeyEvent event) {
								
				if(getSelectedRows().length > 0) directory.updateSelectedBlocksLabel(getSelectedRows());
			}
		});



		accountListScrollPane = new JScrollPane(this);
		accountListScrollPane.setBorder(new LineBorder(backgroundColor, directory.macC64FileOperator.fframe.monitor.convertFloatToRealWidth(0.0015f)));
		accountListScrollPane.setBackground(backgroundColor);
		accountListScrollPane.getVerticalScrollBar().setPreferredSize(new Dimension(directory.macC64FileOperator.fframe.monitor.convertFloatToRealWidth(0.004f), 0));
		accountListScrollPane.getHorizontalScrollBar().setPreferredSize(new Dimension(0, directory.macC64FileOperator.fframe.monitor.convertFloatToRealWidth(0.004f)));
		accountListScrollPane.getVerticalScrollBar().setUI(new BasicScrollBarUI() {   

			@Override 
			protected void configureScrollBarColors(){
				this.trackColor = backgroundColor;
				this.thumbColor = scrollColor;				
			}

			@Override
			protected void	paintDecreaseHighlight(Graphics g) {
			}
			protected void	paintIncreaseHighlight(Graphics g) {
			}

			@Override
			protected JButton createDecreaseButton(int orientation) {
				return createZeroButton();
			}

			@Override    
			protected JButton createIncreaseButton(int orientation) {
				return createZeroButton();
			}

		});



		AbstractAction selectAll = new AbstractAction() {

			public void actionPerformed(ActionEvent evt) {

				try {

					int endRow = directoryTable.getRowCount()-1;
					for(int i=0;i<directoryTable.getRowCount();i++) {

						if(directoryTable.getValueAt(i, 1) == null) {

							endRow = i-1;
							break;
						}
					}	

					directory.directoryTable.setRowSelectionInterval(0, endRow);			
										
				}
				catch (Exception e) {e.printStackTrace(); Toolkit.getDefaultToolkit().beep();}
			}			
		};

		AbstractAction directoryExchangeAction = new AbstractAction() {

			public void actionPerformed(ActionEvent evt) {
				try {

					String p = directory.directoryPath;
					directory.directoryPath = directory.directoryPath;
					directory.directoryPath = p;

					directory.reloadDir();
					directory.reloadDir();

				}
				catch (Exception e) {e.printStackTrace(); Toolkit.getDefaultToolkit().beep();}
			}			
		};		
	}

	private JButton createZeroButton() {
		JButton jbutton = new JButton();
		jbutton.setPreferredSize(new Dimension(0, 0));
		jbutton.setMinimumSize(new Dimension(0, 0));
		jbutton.setMaximumSize(new Dimension(0, 0));
		return jbutton;
	}

	public void columnEvent(int column) {

		directory.activateDirectory();
	}



	public JScrollPane getScrollPane() {

		return accountListScrollPane;
	}

	public void selectRow(String fileName) {

		try {

			for(int i=0;i<directoryTable.getRowCount();i++) {

				String name = ((String)directoryTable.getValueAt(i, 1));
				if(name != null) {

					if(name.compareTo(fileName) == 0) {

						super.changeSelection(i, 1, false, false);	
						directory.clickedDirectoryColumn((String)getValueAt(i, 1));		
						break;
					}
				}
			}			
		}
		catch (Exception e) {e.printStackTrace(); Toolkit.getDefaultToolkit().beep();}

	}

	synchronized public void fillTableWithEmptyRows() {

		// * Leere Zeilen entfernen *
		try {
			for(int i=0;i<tableModel.getRowCount();i++) {			

				if(tableModel.getValueAt(i, 1) == null)  tableModel.removeRow(i);			
			}
		}
		catch(Exception e) {e.printStackTrace();}

		// * Mit leeren Zeilen auffüllen *
		int rows 		= tableModel.getRowCount();
		int rowHeight 	= getRowHeight();
		int tableHeight 	= getTableHeader().getHeight()+(rows*rowHeight);

		while( tableHeight < accountListScrollPane.getViewport().getHeight()) {

			tableModel.addRow(new Object[]{null,null,null,null,null});
			tableHeight+=rowHeight;
		}
	}


	// * synchronized ist wichtig!!! Weil wenn man im Header eine Sortierung aktiviert... die Icons in der Tabelle nicht mehr angezeigt werden! *
	synchronized public void sortTable(JTable directory, int column, String chars) {

		int rows = tableModel.getRowCount();
		for(int i=0;i<rows;i++) {

			if(tableModel.getValueAt(i, column) == null) {

				tableModel.setValueAt(chars, i, column);								
			}
		}							

		tableModel.fireTableDataChanged();

		rows = tableModel.getRowCount();
		for(int i=0;i<rows;i++) {

			if(tableModel.getValueAt(i, column).toString().compareTo(chars) == 0) {

				tableModel.setValueAt(null, i, column);								
			}
		}	
	}


	// * synchronized ist wichtig!!! Weil wenn man im Header eine Sortierung aktiviert... die Icons in der Tabelle nicht mehr angezeigt werden! *
	synchronized public void setDirectoryDesign() {

		try { 

			// * Wenn das Verzeichnis neu geladen wurde, dann Clicked Row Name loeschen, damit beim Click auf das selbe Row gleich der Row Editor gestartet wird! *


			// * Spaltenbreite *

			TableColumnModel tcm = getColumnModel();
			if(tcm.getColumnCount() == 0) return;

			TableColumn column = null;
			for (int i = 0; i < directory.columnNames.length; i++) {

				column = tcm.getColumn(i);
				column.setCellRenderer(new ColoredTableCellRenderer());

				// * Icon Size *
				if (i == 0) 	{
					column.setMinWidth(directory.directoryIconRowWidth); // 5 Pixel Abstand, damit nicht so gequetscht am linken Rand!
					column.setMaxWidth(directory.directoryIconRowWidth); // 5 Pixel Abstand, damit nicht so gequetscht am linken Rand!
				}
				else if (i == 1) 	column.setMinWidth(directory.directoryNameRowMinWidth);
				else if (i == 2) 	column.setMinWidth(directory.directorySizeRowWidth); 				

			}

		}
		catch(Exception e) {e.printStackTrace();}

	}





	boolean theWholeRow = false;

	class ColoredTableCellRenderer extends DefaultTableCellRenderer {

		JImage label = new JImage(); {

			label.setVerticalTextPosition(JLabel.CENTER);
			label.setHorizontalTextPosition(JLabel.CENTER);
			label.setFont(directory.directoryTableFont);
		}

		JLabel textField = new JLabel(); {

			textField.setVerticalTextPosition(JLabel.CENTER);
			textField.setHorizontalTextPosition(JLabel.LEFT);	
			textField.setFont(directory.directoryTableFont);
		}


		public Component getTableCellRendererComponent(JTable table, Object value, boolean selected, boolean focused, int row, int column)	{
			super.getTableCellRendererComponent(table, value, selected, focused, row, column);


			// * Blauen Rahmen bei leeren Zellen deaktivieren (null) *
			setBorder(null);

			setBackground(backgroundColor);


			// * Wenn Cell = null, dann nicht weiter verarbeiten *
			if(value == null) return this;


			if( value instanceof String) {

				if(column == 1 && value.toString().startsWith(".")) theWholeRow = true;

				textField.setForeground(foregroundColor);


				if(column == 2) textField.setHorizontalAlignment(JTextField.CENTER);

				if(column == 1 && value.toString().contains("BLOCKS FREE") == false) 	textField.setText("\"" + value + "\"");
				else																	textField.setText("" + value);

				textField.setIcon(null);

				Color c = backgroundColor;

				if(selected) {

					if(column == 1 && value.toString().contains("BLOCKS FREE") == false) {

						// * Wenn selektiert, dann Hintergrund Blau *
						if(value != null) c = foregroundColor;	 // *  Table Row Color if selected *
						textField.setForeground(backgroundColor);
					}
				}

				textField.setOpaque(true);
				textField.setBackground(c);

				return textField;
			}
			return this;
		}
	}

	// * JTable Header *
	public class TableHeaderRenderer extends JComponent implements TableCellRenderer {

		private final TableCellRenderer baseRenderer;

		JComponent compoment;

		public TableHeaderRenderer(TableCellRenderer baseRenderer) {
			this.baseRenderer = baseRenderer;			
		}

		// * Trennlinie zwischen Table Header und Liste *
		@Override
		public JComponent getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
			JComponent c = (JComponent)baseRenderer.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);


			// * Flat Header Table Border *
			c.setBorder(BorderFactory.createLineBorder(foregroundColor));
			c.setBackground(foregroundColor);

			return c;
		}				
	}
}
