package main;

import java.awt.Color;
import java.awt.Desktop;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;

import ffilechooser.FButton;
import ffilechooser.FFileChooser;
import ffilechooser.FLabel;

public class GUI {

	MacC64FileOperator macC64FileOperator;

	FLabel filenameTextLeft;
	FLabel waveformInfosTextLeft;
	FLabel makerInfoText;

	FButton dirLeftHomeButton;
	FButton dirLeftBackButton;
	FLabel dirLeftLabel;	
	FLabel dirLeftSelectedBlocks;	
	FButton leftNewButton;
	FButton leftRenameButton;
	FButton leftDeleteButton;
	FButton leftFormatButton;
	FButton leftBackupButton;
	FButton leftRunButton;

	FButton dirRightHomeButton;
	FButton dirRightBackButton;
	FLabel dirRightLabel;	
	FLabel dirRightSelectedBlocks;	
	FButton rightNewButton;
	FButton rightRenameButton;
	FButton rightDeleteButton;
	FButton rightFormatButton;
	FButton rightBackupButton;
	FButton rightRunButton;

	FButton arrowLeftButton;
	FButton arrowRightButton;

	FButton exportButton;

	FButton loadButton;
	FButton saveButton;

	Font font;
	Font font2;


	public GUI(MacC64FileOperator macC64FileOperator) {

		this.macC64FileOperator = macC64FileOperator;


		font 	= new Font("Dialog", Font.PLAIN, 12);	

		dirLeftHomeButton = createButton(dirLeftHomeButton, "", new Runnable() {

			@Override
			public void run() {

				macC64FileOperator.switchDirMode(0, MacC64FileOperator.MODE_NORMAL_DIR);
				macC64FileOperator.fileDirLeft.loadDirectory("/Volumes", false);
			}			
		});

		try {
			BufferedImage image = ImageIO.read(MacC64FileOperator.class.getResourceAsStream("/images/home.png"));
			dirLeftHomeButton.setImage(image);			
		} catch (IOException e1) {	e1.printStackTrace();}

		dirRightHomeButton = createButton(dirRightHomeButton, "", new Runnable() {

			@Override
			public void run() {

				macC64FileOperator.switchDirMode(1, MacC64FileOperator.MODE_NORMAL_DIR);
				macC64FileOperator.fileDirRight.loadDirectory("/Volumes", false);
			}			
		});

		try {
			BufferedImage image = ImageIO.read(MacC64FileOperator.class.getResourceAsStream("/images/home.png"));
			dirRightHomeButton.setImage(image);			
		} catch (IOException e1) {	e1.printStackTrace();}

		dirLeftBackButton = createButton(dirLeftBackButton, "", new Runnable() {

			@Override
			public void run() {

				if(macC64FileOperator.leftDirMode == MacC64FileOperator.MODE_D64_DIR ) {

					macC64FileOperator.switchDirMode(0, MacC64FileOperator.MODE_NORMAL_DIR);
				}
				else {

					String path = new File(macC64FileOperator.fileDirLeft.directoryPath).getParent();
					if(path.length() < 1 || path.compareTo("/") == 0) path = "/Volumes";

					macC64FileOperator.fileDirLeft.loadDirectory(path, false);
				}
			}			
		});

		try {
			BufferedImage image = ImageIO.read(MacC64FileOperator.class.getResourceAsStream("/images/arrow_up.png"));
			dirLeftBackButton.setImage(image);			
		} catch (IOException e1) {	e1.printStackTrace();}

		dirRightBackButton = createButton(dirRightBackButton, "", new Runnable() {

			@Override
			public void run() {

				if(macC64FileOperator.rightDirMode == MacC64FileOperator.MODE_D64_DIR ) {

					macC64FileOperator.switchDirMode(1, MacC64FileOperator.MODE_NORMAL_DIR);
				}
				else {

					String path = new File(macC64FileOperator.fileDirRight.directoryPath).getParent();
					if(path.length() < 1 || path.compareTo("/") == 0) path = "/Volumes";

					macC64FileOperator.fileDirRight.loadDirectory(path, false);	
				}
			}			
		});

		try {
			BufferedImage image = ImageIO.read(MacC64FileOperator.class.getResourceAsStream("/images/arrow_up.png"));
			dirRightBackButton.setImage(image);			
		} catch (IOException e1) {	e1.printStackTrace();}


		dirLeftLabel = new FLabel(macC64FileOperator.fframe);
		dirLeftLabel.setText("");
		dirLeftLabel.setFontColor(Color.BLACK);
		dirLeftLabel.setFont(font);
		dirLeftLabel.setFontSize(0.010f);
		dirLeftLabel.setForeground(new Color(0x888888));
		dirLeftLabel.setVisible(true);
		macC64FileOperator.add(dirLeftLabel);

		dirRightLabel = new FLabel(macC64FileOperator.fframe);
		dirRightLabel.setText("");
		dirRightLabel.setFontColor(Color.BLACK);
		dirRightLabel.setFont(font);
		dirRightLabel.setFontSize(0.010f);
		dirRightLabel.setForeground(new Color(0x888888));
		dirRightLabel.setVisible(true);
		macC64FileOperator.add(dirRightLabel);



		dirLeftSelectedBlocks = new FLabel(macC64FileOperator.fframe);
		dirLeftSelectedBlocks.setHorizontalAlignment(JLabel.RIGHT);
		dirLeftSelectedBlocks.setText("[0 Blocks selected]");
		dirLeftSelectedBlocks.setFontColor(Color.BLACK);
		dirLeftSelectedBlocks.setFont(font);
		dirLeftSelectedBlocks.setFontSize(0.011f);
		dirLeftSelectedBlocks.setForeground(new Color(0x888888));
		dirLeftSelectedBlocks.setVisible(false);
		macC64FileOperator.add(dirLeftSelectedBlocks);

		dirRightSelectedBlocks = new FLabel(macC64FileOperator.fframe);
		dirRightSelectedBlocks.setText("[0 Blocks selected]");
		dirRightSelectedBlocks.setFontColor(Color.BLACK);
		dirRightSelectedBlocks.setFont(font);
		dirRightSelectedBlocks.setFontSize(0.011f);
		dirRightSelectedBlocks.setForeground(new Color(0x888888));
		dirRightSelectedBlocks.setVisible(false);
		macC64FileOperator.add(dirRightSelectedBlocks);


		leftNewButton = createButton(leftNewButton, "NEW", new Runnable() {

			@Override
			public void run() {

				macC64FileOperator.newFile(0);
			}			
		});

		rightNewButton = createButton(rightNewButton, "NEW", new Runnable() {

			@Override
			public void run() {

				macC64FileOperator.newFile(1);
			}			
		});

		leftDeleteButton = createButton(leftDeleteButton, "DEL", new Runnable() {

			@Override
			public void run() {

				macC64FileOperator.deleteFile(0);
			}			
		});

		rightDeleteButton = createButton(rightDeleteButton, "DEL", new Runnable() {

			@Override
			public void run() {
				macC64FileOperator.deleteFile(1);
			}			
		});

		leftFormatButton = createButton(leftFormatButton, "FORMAT", new Runnable() {

			@Override
			public void run() {

				macC64FileOperator.formatDisk(0);
			}			
		});

		rightFormatButton = createButton(rightFormatButton, "FORMAT", new Runnable() {

			@Override
			public void run() {
				macC64FileOperator.formatDisk(1);				
			}			
		});


		leftRenameButton = createButton(leftRenameButton, "RENAME", new Runnable() {

			@Override
			public void run() {
				macC64FileOperator.renameFile(0, false);
			}			
		});

		rightRenameButton = createButton(rightRenameButton, "RENAME", new Runnable() {

			@Override
			public void run() {
				macC64FileOperator.renameFile(1, false);
			}			
		});


		leftBackupButton = createButton(leftBackupButton, "RESTORE", new Runnable() {

			public void run() {
				macC64FileOperator.backupDisk(0);
			};
		});
		leftBackupButton.setVisible(true);

		rightBackupButton = createButton(rightBackupButton, "RESTORE", new Runnable() {

			public void run() {
				macC64FileOperator.backupDisk(1);
			};
		});
		rightBackupButton.setVisible(true);



		leftRunButton = createButton(leftRunButton, "RUN", new Runnable() {

			public void run() {
				macC64FileOperator.runEmu(0);				
			};
		});
		leftRunButton.setVisible(true);


		rightRunButton = createButton(rightRunButton, "RUN", new Runnable() {

			public void run() {
				macC64FileOperator.runEmu(1);				
			};
		});
		rightRunButton.setVisible(true);


		arrowLeftButton = createButton(arrowLeftButton, "", new Runnable() {

			@Override
			public void run() {
				macC64FileOperator.copySelectedFiles(1);		
			}			
		});

		try {
			BufferedImage image = ImageIO.read(MacC64FileOperator.class.getResourceAsStream("/images/arrow_left.png"));
			arrowLeftButton.setImage(image);

		} catch (Exception e1) {
			e1.printStackTrace();
		}

		arrowRightButton = createButton(arrowRightButton, "", new Runnable() {

			@Override
			public void run() {
				macC64FileOperator.copySelectedFiles(0);			
			}			
		});

		try {
			BufferedImage image = ImageIO.read(MacC64FileOperator.class.getResourceAsStream("/images/arrow_right.png"));
			arrowRightButton.setImage(image);

		} catch (IOException e1) {
			e1.printStackTrace();
		}


		loadButton = createButton(loadButton, "Load", new Runnable() {
			public void run() {
				try {

					float w = 0.35f;
					float h = 0.4f;

					macC64FileOperator.fileChooser = new FFileChooser(this.getClass(), "Files selection...", FFileChooser.TYPE_LOAD, macC64FileOperator.oldStyle);
					macC64FileOperator.fileChooser.setBounds(0.5f-w/2,0.5f-h/2,w,h);
					macC64FileOperator.fileChooser.setMinimumSize(0.25f, 0.30f);

					int returnVal = macC64FileOperator.fileChooser.show();															
					if (returnVal == 1) {

						File f = macC64FileOperator.fileChooser.getSelectedFile();
						if(f.isFile() && f.getName().toLowerCase().contains(".d64") == true) {

							macC64FileOperator.c64DirLeft.loadDirectory(f.getAbsolutePath(), false);
						}
					}

					macC64FileOperator.fileChooser = null;
				}
				catch(Exception e) {}
			};
		});
		loadButton.setVisible(false);

		saveButton = createButton(saveButton, "Save", new Runnable() {
			public void run() {

				try {

					int dialogResult = JOptionPane.showConfirmDialog (null, "Are you sure to override the 'original' Audio File?","Warning", JOptionPane.YES_NO_OPTION,JOptionPane.PLAIN_MESSAGE);
					if(dialogResult == JOptionPane.YES_OPTION) {


					}

				}
				catch(Exception e) {}
			};
		});
		saveButton.setVisible(false);


		makerInfoText = new FLabel(macC64FileOperator.fframe);
		makerInfoText.setAlignmentX(SwingConstants.RIGHT);
		makerInfoText.setText("Design by Matthias");
		makerInfoText.setFontColor(Color.BLACK);
		makerInfoText.setFont(font);
		makerInfoText.setFontSize(0.010f);
		makerInfoText.setForeground(new Color(0x888888));
		makerInfoText.setVisible(true);
		macC64FileOperator.add(makerInfoText);

		SwingUtilities.invokeLater(new Runnable() {
			public void run() {

				updateGUI();
			}});
	}

	private FButton createButton(FButton button, String text, Runnable runnable) {

		button = new FButton(macC64FileOperator.fframe);
		button.putClientProperty( "JButton.buttonType", "textured" );
		button.setText(text);
		button.setFont(font);
		button.setFontSize(0.0115f);
		button.setBorderColor(new Color(160,160,160));
		button.addActionListener( new ActionListener() {

			public void actionPerformed(ActionEvent e) {

				SwingUtilities.invokeLater(runnable);
			}
		} );
		macC64FileOperator.add(button);	

		return button;
	}


	public void updateGUI() {

		float yPos = 0.022f;
		float height = 0.020f;

		macC64FileOperator.c64DirLeft.setBounds(0.016f, 0.075f, 0.186f, 0.28f);		
		macC64FileOperator.fileDirLeft.setBounds(0.016f, 0.075f, 0.186f, 0.28f);

		macC64FileOperator.c64DirRight.setBounds(0.24f, 0.075f, 0.186f, 0.28f);
		macC64FileOperator.fileDirRight.setBounds(0.24f, 0.075f, 0.186f, 0.28f);


		dirLeftHomeButton.setBounds(0.0153f, 0.049f, 0.012f, height);
		dirLeftBackButton.setBounds(0.0283f, 0.049f, 0.012f, height);
		dirLeftLabel.setBounds(0.0443f, 0.0515f, 0.15f, height);
		dirLeftSelectedBlocks.setBounds(0.132f, 0.36f, 0.07f, height);

		leftFormatButton.setBounds(	0.01525f, 	yPos, 0.033f, height);
		leftBackupButton.setBounds(	0.0511f, 	yPos, 0.033f, height);
		leftNewButton.setBounds(		0.0868f, 	yPos, 0.025f, height);
		leftRenameButton.setBounds(	0.1152f, 	yPos, 0.035f, height);
		leftDeleteButton.setBounds(	0.1529f, 	yPos, 0.021f, height);
		leftRunButton.setBounds(		0.1762f, 		yPos, 0.025f, height);

		dirRightHomeButton.setBounds(0.2395f, 0.049f, 0.012f, height);
		dirRightBackButton.setBounds(0.2525f, 0.049f, 0.012f, height);
		dirRightLabel.setBounds(0.2685f, 0.0515f, 0.15f, height);
		dirRightSelectedBlocks.setBounds(0.240f, 0.36f, 0.7f, height);

		rightFormatButton.setBounds(	0.2396f, 	yPos, 0.033f, height);
		rightBackupButton.setBounds(	0.2755f, 	yPos, 0.033f, height);
		rightNewButton.setBounds(	0.3115f, 	yPos, 0.025f, height);
		rightRenameButton.setBounds(	0.3395f, 	yPos, 0.035f, height);
		rightDeleteButton.setBounds(	0.3767f, 	yPos, 0.021f, height);
		rightRunButton.setBounds(	0.400f, 		yPos, 0.025f, height);


		arrowLeftButton.setBounds(0.205f, 0.21f, 0.013f, height);
		arrowRightButton.setBounds(0.2235f, 0.21f, 0.013f, height);

		makerInfoText.setBounds(0.375f, 0.4f, 0.2f, 0.07f);

		loadButton.setBounds(0.015f, 0.018f, 0.033f, height);
		saveButton.setBounds(0.172f, 0.018f, 0.03f, height);

		macC64FileOperator.updateDirLabels();

	}	


	public void updateButtons() {

		MacC64FileOperator main = macC64FileOperator;



		// * Format *				
		if((main.leftDirMode == MacC64FileOperator.MODE_D64_DIR )) 		leftFormatButton.setEnabled(true);
		else																		leftFormatButton.setEnabled(false);																							
		if((main.rightDirMode == MacC64FileOperator.MODE_D64_DIR )) 	rightFormatButton.setEnabled(true);
		else																		rightFormatButton.setEnabled(false);																							

		
		
		// * Backup *
		if(main.leftDirMode == MacC64FileOperator.MODE_D64_DIR ) 														leftBackupButton.setEnabled(true);
		else																		leftBackupButton.setEnabled(false);																							
		if(main.rightDirMode == MacC64FileOperator.MODE_D64_DIR ) 														rightBackupButton.setEnabled(true);
		else																		rightBackupButton.setEnabled(false);
		
		if(main.leftDirMode == MacC64FileOperator.MODE_NORMAL_DIR && (main.rightDirMode == MacC64FileOperator.MODE_D64_DIR )) {
			
			String[] selectedFiles = main.fileDirLeft.getSelectedFiles();			
			if(selectedFiles != null && selectedFiles.length == 1) {
				
				String selectedFilePath = main.fileDirLeft.directoryPath + "/" + selectedFiles[0];							
				if(selectedFilePath.toLowerCase().contains(".d64")
						|| selectedFilePath.toLowerCase().contains(".g64")
						|| selectedFilePath.toLowerCase().contains(".nib")
						) {	

					leftBackupButton.setEnabled(true);					
				}
			}
		}
		
		if(main.rightDirMode == MacC64FileOperator.MODE_NORMAL_DIR && (main.leftDirMode == MacC64FileOperator.MODE_D64_DIR)) {
			
			String[] selectedFiles = main.fileDirRight.getSelectedFiles();			
			if(selectedFiles != null && selectedFiles.length == 1) {
				
				String selectedFilePath = main.fileDirRight.directoryPath + "/" + selectedFiles[0];							
				if(selectedFilePath.toLowerCase().contains(".d64")
						|| selectedFilePath.toLowerCase().contains(".g64")
						|| selectedFilePath.toLowerCase().contains(".nib")
						) {	

					rightBackupButton.setEnabled(true);					
				}
			}
		}
		
		
		
		// * New *
		if(main.leftDirMode == MacC64FileOperator.MODE_NORMAL_DIR)				leftNewButton.setEnabled(true);
		else																		leftNewButton.setEnabled(false);		
		if(main.rightDirMode == MacC64FileOperator.MODE_NORMAL_DIR)				rightNewButton.setEnabled(true);
		else																		rightNewButton.setEnabled(false);		

		
		
		// * Rename *
		if(main.leftDirMode == MacC64FileOperator.MODE_NORMAL_DIR) {
			
			String[] selectedFiles = main.fileDirLeft.getSelectedFiles();			
			if(selectedFiles != null && selectedFiles.length == 1) {
				
				String selectedFilePath = main.fileDirLeft.directoryPath + "/" + selectedFiles[0];							
				if(selectedFilePath.toLowerCase().contains(".del") 
						|| selectedFilePath.toLowerCase().contains(".seq")
						|| selectedFilePath.toLowerCase().contains(".prg")
						|| selectedFilePath.toLowerCase().contains(".usr")
						|| selectedFilePath.toLowerCase().contains(".rel")
						|| selectedFilePath.toLowerCase().contains(".p00")
						|| selectedFilePath.toLowerCase().contains(".t64")	
						|| selectedFilePath.toLowerCase().contains(".d64")
						|| selectedFilePath.toLowerCase().contains(".g64")
						|| selectedFilePath.toLowerCase().contains(".nib")
						) {	

					leftRenameButton.setEnabled(true);
					leftDeleteButton.setEnabled(true);
					leftRunButton.setEnabled(true);
				}
				else {
					
					leftRenameButton.setEnabled(false);
					leftDeleteButton.setEnabled(false);
					leftRunButton.setEnabled(false);
				}
			}	
			else {
				
				leftRenameButton.setEnabled(false);
				if(selectedFiles != null && selectedFiles.length < 1) leftDeleteButton.setEnabled(false);
				leftRunButton.setEnabled(false);
			}
		}

		if(main.rightDirMode == MacC64FileOperator.MODE_NORMAL_DIR) {

			String[] selectedFiles = main.fileDirRight.getSelectedFiles();
			if(selectedFiles != null && selectedFiles.length == 1) {
								
				String selectedFilePath = main.fileDirRight.directoryPath + "/" + selectedFiles[0];							
				if(selectedFilePath.toLowerCase().contains(".del") 
						|| selectedFilePath.toLowerCase().contains(".seq")
						|| selectedFilePath.toLowerCase().contains(".prg")
						|| selectedFilePath.toLowerCase().contains(".usr")
						|| selectedFilePath.toLowerCase().contains(".rel")
						|| selectedFilePath.toLowerCase().contains(".p00")
						|| selectedFilePath.toLowerCase().contains(".t64")	
						|| selectedFilePath.toLowerCase().contains(".d64")
						|| selectedFilePath.toLowerCase().contains(".g64")
						|| selectedFilePath.toLowerCase().contains(".nib")
						) {	

					rightRenameButton.setEnabled(true);
					rightDeleteButton.setEnabled(true);
					rightRunButton.setEnabled(true);
				}
				else {
					
					rightRenameButton.setEnabled(false);
					rightDeleteButton.setEnabled(false);
					rightRunButton.setEnabled(false);
				}
			}	
			else {
				
				rightRenameButton.setEnabled(false);
				if(selectedFiles != null && selectedFiles.length < 1) rightDeleteButton.setEnabled(false);
				rightRunButton.setEnabled(false);
			}
		}
		
		
		if(main.leftDirMode == MacC64FileOperator.MODE_D64_DIR) {

			String[] selectedFiles = main.c64DirLeft.getSelectedFiles();
			if(selectedFiles != null && selectedFiles.length == 1) 	{
				
				leftRenameButton.setEnabled(true);
				leftDeleteButton.setEnabled(true);
				leftRunButton.setEnabled(true);
			}
			else	 {
				
				leftRenameButton.setEnabled(false);
				if(selectedFiles.length < 1) leftDeleteButton.setEnabled(false);
				leftRunButton.setEnabled(false);
			}
		}
		
		if(main.rightDirMode == MacC64FileOperator.MODE_D64_DIR) {

			String[] selectedFiles = main.c64DirRight.getSelectedFiles();
			if(selectedFiles != null && selectedFiles.length == 1) {
				
				rightRenameButton.setEnabled(true);
				rightDeleteButton.setEnabled(true);
				rightRunButton.setEnabled(true);
			}
			else	 {
				
				rightRenameButton.setEnabled(false);
				if(selectedFiles.length < 1) rightDeleteButton.setEnabled(false);
				rightRunButton.setEnabled(false);
			}
		}
		

	}


}
