package main;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.GraphicsEnvironment;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.file.CopyOption;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.SwingUtilities;

public class Helper {

	MacC64FileOperator macC64FileOperator;
	
	
	
	
	public Helper(MacC64FileOperator macC64FileOperator) {

		this.macC64FileOperator = macC64FileOperator;
	}

	public void loadFont() {

		try {
			

			InputStream in = getClass().getResourceAsStream("/fonts/C64.ttf");
			macC64FileOperator.baseFont = Font.createFont(Font.TRUETYPE_FONT, in);
			GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
			ge.registerFont(macC64FileOperator.baseFont);
			
			System.out.println("OK!");
		} 
		catch (Exception e) {			
			e.printStackTrace();
		} 
	}


	public void copyFile(File source, File target) {
		try {Files.copy(Paths.get(source.getAbsolutePath()), Paths.get(target.getAbsolutePath()), StandardCopyOption.REPLACE_EXISTING);} catch (Exception e) { e.printStackTrace();}
	}

	public void deleteFile(File f) throws IOException {
		if (f.isDirectory()) {
			for (File c : f.listFiles())
				deleteFile(c);
		}
		if (!f.delete())
			throw new FileNotFoundException("Failed to delete file: " + f);
	}


	public String getExtension(File file)	{

		String fileExtension = "";
		String fileName = file.getName();

		if(fileName.contains(".") && fileName.lastIndexOf(".")!= 0)	fileExtension = fileName.substring(fileName.lastIndexOf(".") +1);
		return fileExtension;
	}	

	public BufferedImage getBitmapFromAssets(String filePath) {

		try {
			InputStream in = ClassLoader.class.getResourceAsStream("/" + filePath);			
			return ImageIO.read(in);
		} 
		catch (Exception e) {	e.printStackTrace(); }	
		return null;
	}	
	
	
	public String createTempPRG(String sourceFile, String targetFile) {
			
		String destFile = targetFile;

		try {

			if(new File(sourceFile).exists() == false) return null;
			
			if(new File(destFile).exists()) new File(destFile).delete();				
			

			FileChannel source = null;
			FileChannel destination = null;
			try {
				source = new FileInputStream(sourceFile).getChannel();
				destination = new FileOutputStream(destFile).getChannel();

				// * In Haeppchen Uebertragen (maximal 64 MB je Happen) *
				int maxCount = (64 * 1024 * 1024) - (32 * 1024);
				long size = source.size();
				long position = 0;

				if(sourceFile.toLowerCase().endsWith(".prg")) {
					
					position = 0;
					
					// * File kopieren *
					while (position < size) position += source.transferTo(position, maxCount, destination);	
				}
				else if(sourceFile.toLowerCase().endsWith(".p00")) 	{
										
					position = 26;
					
					// * File kopieren *
					while (position < size) position += source.transferTo(position, maxCount, destination);
				}
				else if(sourceFile.toLowerCase().endsWith(".t64")) {

					// * hier wird das T64 File in ein PRG umgewandelt *

					File assembled =  new File(sourceFile); 
					RandomAccessFile raf = new RandomAccessFile(assembled, "r"); 

					raf.seek(0x48);					
					int ch4 = raf.read();
					int ch3 = raf.read();
					int ch2 = raf.read();
					int ch1 = raf.read();
					
					if ((ch1 | ch2 | ch3 | ch4) < 0) return "NULL";
					else {

						int offset = ((ch1 << 24) + (ch2 << 16) + (ch3 << 8) + (ch4 << 0));
						
						raf.seek(0x42);					
						int startAddress = (raf.read() & 0xff) + (raf.read() << 8);

						raf.seek(0x44);					
						int endAddress = (raf.read() & 0xff) + (raf.read() << 8);		    		
						int len = endAddress-startAddress;				    

						raf.seek(offset);					
						int b0 = raf.read();
						int b1 = raf.read();						
						if(b0 == 0) 				offset++;
						if(b0 == 0 && b1 == 0) 	offset += 2;
						
						
						// * File kopieren *
						while (position < len) position += source.transferTo(position, maxCount, destination);	
					}

					raf.close();		

				}	
				else if(sourceFile.toLowerCase().endsWith(".crt")) {

					File assembled =  new File(sourceFile); 
					RandomAccessFile raf = new RandomAccessFile(assembled, "r"); 


					boolean chip = false;
					int pos = 0x40; // * CHIP:  Block 0 vorhanden? *
					int len = 0;
					int blockCounter = 0;

					do {

						chip = false;
						raf.seek(pos); 


						byte[] header = new byte[16];
						for(int i=0;i<header.length;i++) header[i] = (byte)raf.read();


						if(header[0] == 0x43 && header[1] == 0x48 && header[2] == 0x49 && header[3] == 0x50) chip = true;
						if(chip == false) break;


						int load_address = (header[0xc] << 8) + header[0xd];
						int length = (header[4] << 24) + (header[5] << 16) + (header[6] << 8) + header[7] - 16;


						if(blockCounter == 0) {  // Vor Block 0 die Ladeadresse schreiben...

							int zielAdresseImSpeicher = 0x8000;
							//int zielAdresseImSpeicher = 0x2000;

							ByteBuffer b = ByteBuffer.allocate(2);
							//b.array()[0] = (byte) (load_address & 0xff);;
							//b.array()[1] = (byte) (load_address >> 8);	
							b.array()[0] = (byte) (zielAdresseImSpeicher & 0xff);;
							b.array()[1] = (byte) (zielAdresseImSpeicher >> 8);	

							destination.write(b);
						}


						pos += 0x10;
						raf.seek(pos);

						// * Block Kopieren *
						source.transferTo(pos, length, destination);			


						blockCounter++;
						pos += length;
					}
					while(chip);


					raf.close();								


					if(source != null) source.close();						
					if(destination != null) 	destination.close();

					return destFile;						

				}
				
			}

			finally {
				if(source != null) {
					source.close();
				}
				if(destination != null) {
					destination.close();
				}
			}			

			return destFile;
		}
		catch(Exception e) {}

		return null;
	}
}
