package main;
import java.awt.Color;
import java.awt.Desktop;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;
import java.util.Vector;
import java.util.prefs.Preferences;

import javax.swing.JComponent;
import javax.swing.JLayeredPane;
import javax.swing.SwingUtilities;

import d64.D64;
import d64.DirEntry;
import ffilechooser.FFileChooser;
import ffilechooser.FFrame;
import ffilechooser.FFrameListener;


public class MacC64FileOperator extends JLayeredPane {

	boolean log = true;


	// * Preferences *
	static Preferences prefs = Preferences.userNodeForPackage(MacC64FileOperator.class);

	FFileChooser fileChooser;	
	boolean oldStyle = false;
	String[] favorites;


	public static FFrame fframe;

	static float frameWidth = 0.45f;
	static float frameHeight = 0.45f;

	static Font baseFont;


	String systemPassword = "";

	public static void main(String args[]) {


		fframe = new FFrame(new FFrameListener() {
			@Override
			public void monitorResolutionChanged() {}
			@Override
			public void frameResized() {}
			@Override
			public void frameMoved() {}
		});

		fframe.addWindowListener(new WindowAdapter()	{
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});

		fframe.setTitle("C64 File Operator");
		fframe.setResizable(false);

		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();

		fframe.setSize(frameWidth, frameHeight);
		fframe.setLocation(d.width/2-fframe.monitor.convertFloatToRealWidth(frameWidth/2), d.height/2-fframe.monitor.convertFloatToRealHeight(frameHeight/2));

		// * Hide Toolbar *
		fframe.setUndecorated(false);

		JComponent newContentPane = new MacC64FileOperator();
		newContentPane.setOpaque(true); //content panes must be opaque
		fframe.setContentPane(newContentPane);
		fframe.setForeground(Color.WHITE);
		fframe.setVisible(true);
		fframe.setLayout(null);
		fframe.setBackground(new Color(238,238,238));
		fframe.setFocusable(true);
						
		
	}



	MacC64FileOperator macC64FileOperator;

	Helper helper;

	String internalPath;

	public C64Dir c64DirLeft;
	public C64Dir c64DirRight;
	public FileDir fileDirLeft;
	public FileDir fileDirRight;

	public final static int MODE_NORMAL_DIR 		= 0;
	public final static int MODE_D64_DIR 		= 1;

	public int leftDirMode  	= MODE_NORMAL_DIR;
	public int rightDirMode  = MODE_NORMAL_DIR;


	String leftXUM1541File = "";
	String rightXUM1541File = "";

	GUI gui;


	Popup popup;

	public MacC64FileOperator() {

		this.macC64FileOperator = this;

		internalPath = System.getProperty("user.dir");  //  Mac App Path:  /Contents/Java

		helper = new Helper(this);
		helper.loadFont();

		popup = new Popup(this);

		
		
		String[] t = new String[1];
		t[0] = new String(""); 


		c64DirLeft = new C64Dir(this, 0, t, "C64");
		c64DirLeft.setHeaderTitle("TEST DISK", "00");
		c64DirLeft.setVisible(false);
		add(c64DirLeft);			
		c64DirRight = new C64Dir(this, 1, t, "C64");
		c64DirRight.setHeaderTitle("TEST DISK", "00");
		c64DirRight.setVisible(false);
		add(c64DirRight);	

		fileDirLeft = new FileDir(this,0, t, "Dialog","pref_left");
		fileDirLeft.setVisible(true);		
		add(fileDirLeft);	
		fileDirRight = new FileDir(this, 1, t, "Dialog","pref_right");
		fileDirRight.setVisible(true);		
		add(fileDirRight);	

		gui = new GUI(this);	

		popup.close();

		gui.updateButtons();

	}

	public String shellCommand(String command, boolean errorOutputOnly) {

	//	System.out.println(command);
		
		String output = "";

		try {

			Runtime runtime = Runtime.getRuntime();
			String[] args = new String[3];

			if(systemPassword.compareTo("") == 0)  	args = new String[] { "/bin/bash", "-c", command};
			else 									args = new String[] {"/bin/bash","-c","echo "+systemPassword+"| sudo -S " + command};

			final Process proc = runtime.exec(args);


			String line = "";

			// * Input * 
			InputStream inputStream = proc.getInputStream();
			BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream, "UTF-8"));
			while ((line = reader.readLine()) != null) {if(line != null) output += line; }

			// * Errors * 
			InputStream inputErrStream = proc.getErrorStream();
			BufferedReader reader2 = new BufferedReader(new InputStreamReader(inputErrStream, "UTF-8"));
			while ((line = reader2.readLine()) != null) {if(line != null) output += line; }

		} catch (Exception ex) {
			System.out.println("exception : " + ex.getMessage());
			ex.printStackTrace();
		}

		return output;
	}


	public void switchDirMode(int switchSide, int switchToMode) {

		SwingUtilities.invokeLater(new Runnable() {
			public void run() {

				FileDir fileDir 	= fileDirLeft;
				C64Dir c64Dir 	= c64DirLeft;

				if(switchSide == 1) {

					fileDir 	= fileDirRight;
					c64Dir 	= c64DirRight;
				}

				if(switchToMode == MODE_NORMAL_DIR) {

					if(switchSide == 0) {

						gui.leftBackupButton.setText("RESTORE");
						leftDirMode = MODE_NORMAL_DIR;
						gui.dirLeftSelectedBlocks.setVisible(false);
					}
					else	 {
						gui.rightBackupButton.setText("RESTORE");
						rightDirMode = MODE_NORMAL_DIR;
						gui.dirRightSelectedBlocks.setVisible(false);
					}

					fileDir.setVisible(true);		
					c64Dir.setVisible(false);
				}

				if(switchToMode == MODE_D64_DIR) {

					if(switchSide == 0) {

						gui.leftBackupButton.setText("BACKUP");
						leftDirMode = MODE_D64_DIR;
						gui.dirLeftSelectedBlocks.setVisible(true);
					}
					else	{

						gui.rightBackupButton.setText("BACKUP");
						rightDirMode = MODE_D64_DIR;
						gui.dirRightSelectedBlocks.setVisible(true);
					}

					c64Dir.loadDirectory(fileDir.filePath, false);

					fileDir.setVisible(false);		
					c64Dir.setVisible(true);					
					c64Dir.directory.directoryTable.requestFocus();  // * Needed for KEY A (Select All) after switching to D64 View * 
				}

				

				if(macC64FileOperator.gui != null) macC64FileOperator.gui.updateButtons();
			}});

		updateDirLabels();

	}

	private void updateDir(int sourceSide) {

		SwingUtilities.invokeLater(new Runnable() {
			public void run() {

				if(sourceSide == 0) { // * Source: Left, update Right Side *

					if(rightDirMode == MODE_NORMAL_DIR) 		fileDirRight.reloadDir();
					if(rightDirMode == MODE_D64_DIR) 		c64DirRight.reloadDir();
					
				}
				else { // * Source: Right, update elft Side *

					if(leftDirMode == MODE_NORMAL_DIR) 		fileDirLeft.reloadDir();
					if(leftDirMode == MODE_D64_DIR) 		c64DirLeft.reloadDir();
												
				}					
			}});
	}	

	public void updateDirLabels() {

		SwingUtilities.invokeLater(new Runnable() {
			public void run() {

				try {

					if(leftDirMode == MODE_NORMAL_DIR) 		gui.dirLeftLabel.setText(fileDirLeft.directoryPath);
					if(leftDirMode == MODE_D64_DIR) 			gui.dirLeftLabel.setText("D64:  " + new File(fileDirLeft.filePath).getName().replace(".d64", "").replace(".D64", ""));
					
					if(rightDirMode == MODE_NORMAL_DIR) 		gui.dirRightLabel.setText(fileDirRight.directoryPath);
					if(rightDirMode == MODE_D64_DIR) 		gui.dirRightLabel.setText("D64:  " + new File(fileDirRight.filePath).getName().replace(".d64", "").replace(".D64", ""));
					

					if(gui == null) return;  // * at first call, GUI does not exist *
					
					macC64FileOperator.repaint();

				}
				catch(Exception e) {
				}
			}});
	}

	
	public void formatDisk(int sourceSide) {

		int sourceMode = leftDirMode;
		if(sourceSide == 1) sourceMode = rightDirMode;

		if(sourceMode == MODE_D64_DIR) 			formatD64Disk(sourceSide);
		else {

			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					popup.close();		
				}});
		}
	}

	public void formatD64Disk(int sourceSide) {

		Thread T=new Thread(new Runnable() {

			@Override
			public void run() {

				try {

					boolean reply = popup.question("Are you sure you want to Format?");	
					if(!reply) return;

					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							popup.hint("Formating Disk, please wait...");
						}});


					C64Dir sourceC64Dir 					= c64DirLeft;			
					if(sourceSide == 1) sourceC64Dir 	= c64DirRight;

					// * Override the D64 Filfe with the "asset/EMPTY.D64" File *
					InputStream link = ClassLoader.class.getResourceAsStream("/EMPTY.d64");
					Path path = Paths.get(sourceC64Dir.directoryPath);

					Files.copy(link, path, StandardCopyOption.REPLACE_EXISTING);
					sourceC64Dir.reloadDir();

					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							popup.close();		
						}});
				}
				catch(Exception e) {e.printStackTrace();}
			}
		} );
		T.start();	
	}

	

	void backupDisk(int sourceSide) {

		Thread T = new Thread(new Runnable() {
			@Override
			public void run() {

				try {

					int sourceMode 			= leftDirMode;
					int targetMode 			= rightDirMode;									
					FileDir sourceFileDir 	= fileDirLeft;
					FileDir tragetFileDir 	= fileDirRight;
					C64Dir sourceC64Dir 		= c64DirLeft;
					C64Dir targetC64Dir 		= c64DirRight;

					if(sourceSide == 1) {

						sourceMode 		= rightDirMode;
						targetMode 		= leftDirMode;										
						sourceFileDir 	= fileDirRight;
						tragetFileDir 	= fileDirLeft;
						sourceC64Dir 	= c64DirRight;
						targetC64Dir 	= c64DirLeft;
					}

					boolean updateDirsAfterCommand = false;

					// * Copy "D64" Disk to ... ".d64" file *
					if(sourceMode == MODE_D64_DIR && targetMode == MODE_NORMAL_DIR) {		

						File targetFile = new File(tragetFileDir.directoryPath + "/" + new File(sourceC64Dir.directoryPath).getName());

						if(targetFile.exists()) {

							boolean reply = popup.question("\"" + targetFile.getName() + "\"\nstill exist!");	
							if(!reply) return;
						}


						boolean reply = popup.question("Are you sure you want to Backup to:\n " + targetFile.getName() + " ?");	
						if(!reply) return;

						SwingUtilities.invokeLater(new Runnable() {
							public void run() {
								popup.hint("Creating Backup File:\n" + targetFile);
							}});

						updateDirsAfterCommand = true;
						Files.copy(new File(sourceC64Dir.directoryPath).toPath(), targetFile.toPath(), StandardCopyOption.REPLACE_EXISTING);
					}		


					// * Copy "D64" Disk to ... "D64" Disk *
					else if(sourceMode == MODE_D64_DIR && targetMode == MODE_D64_DIR) {		

						String extension = helper.getExtension(new File(targetC64Dir.directoryPath));
						String targetD64DiskName = new File(targetC64Dir.directoryPath).getName().replaceAll("."+extension, "");

						boolean reply = popup.question("Are you sure you want to override Disk:\n\"" + targetD64DiskName+"\"");	
						if(!reply) return;

						SwingUtilities.invokeLater(new Runnable() {
							public void run() {
								popup.hint("Backuping Disk, please wait...");
							}});


						updateDirsAfterCommand = true;
						Files.copy(new File(sourceC64Dir.directoryPath).toPath(), new File(targetC64Dir.directoryPath).toPath(), StandardCopyOption.REPLACE_EXISTING);
					}		
					
					// * Copy .d64 File D64 Disk *
					else if((sourceMode == MODE_NORMAL_DIR) && targetMode == MODE_D64_DIR) {		

						boolean reply = popup.question("Are you sure you want to override Disk:\n" + new File(targetC64Dir.directoryPath).getName());	
						if(!reply) return;


						String[] selectedFiles = sourceFileDir.getSelectedFiles();
						if(selectedFiles.length == 1 && selectedFiles[0].toLowerCase().contains(".d64")) {

							String selectedFilePath = sourceFileDir.directoryPath + "/" + selectedFiles[0];							

							SwingUtilities.invokeLater(new Runnable() {
								public void run() {
									popup.hint("Backuping Disk, please wait...");
								}});

							updateDirsAfterCommand = true;
							Files.copy(new File(selectedFilePath).toPath(), new File(targetC64Dir.directoryPath).toPath(), StandardCopyOption.REPLACE_EXISTING);
						}								
					}

				
					

					
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							popup.close();		
						}});

					if(updateDirsAfterCommand) updateDir(sourceSide);
				} 
				catch (Exception e) {e.printStackTrace();}
			}
		} );
		T.start();	
	}	

	public void newFile(int sourceSide) {

		Thread T=new Thread(new Runnable() {

			@Override
			public void run() {

				try {

					int sourceMode 			= leftDirMode;
					int targetMode 			= rightDirMode;									
					FileDir sourceFileDir 	= fileDirLeft;
					FileDir tragetFileDir 	= fileDirRight;
					C64Dir sourceC64Dir 		= c64DirLeft;
					C64Dir targetC64Dir 		= c64DirRight;

					if(sourceSide == 1) {

						sourceMode 		= rightDirMode;
						targetMode 		= leftDirMode;										
						sourceFileDir 	= fileDirRight;
						tragetFileDir 	= fileDirLeft;
						sourceC64Dir 	= c64DirRight;
						targetC64Dir		= c64DirLeft;
					}

					if(sourceMode == MODE_NORMAL_DIR) {

						String diskName = popup.inputField("Create a new .D64 File", "EMPTY", true);
						if(diskName != null) {

							String filePath = sourceFileDir.directoryPath + "/" + diskName + ".d64";

							File f = new File(filePath);
							f.createNewFile();

							sourceC64Dir.loadDirectory(filePath, false);

							// * Override the D64 File with the "asset/EMPTY.D64" File *
							InputStream link = ClassLoader.class.getResourceAsStream("/EMPTY.d64");
							Path path = Paths.get(sourceC64Dir.directoryPath);
							Files.copy(link, path, StandardCopyOption.REPLACE_EXISTING);

							sourceC64Dir.d64.setDiskNameAndID(diskName, "   2A");
							sourceC64Dir.reloadDir();

							sourceFileDir.reloadDir();
						}
					}
				} 
				catch (Exception e) {
					e.printStackTrace();
				}
			}
		} );
		T.start();	
	}

	public void renameFile(int sourceSide, boolean diskTitle) {

		Thread T=new Thread(new Runnable() {

			@Override
			public void run() {

				try {

					int sourceMode 			= leftDirMode;
					int targetMode 			= rightDirMode;									
					FileDir sourceFileDir 	= fileDirLeft;
					FileDir tragetFileDir 	= fileDirRight;
					C64Dir sourceC64Dir 		= c64DirLeft;
					C64Dir targetC64Dir 		= c64DirRight;

					if(sourceSide == 1) {

						sourceMode 		= rightDirMode;
						targetMode 		= leftDirMode;										
						sourceFileDir 	= fileDirRight;
						tragetFileDir 	= fileDirLeft;
						sourceC64Dir 	= c64DirRight;
						targetC64Dir		= c64DirLeft;
					}

					if(diskTitle) {

						String[] diskNameAndID = popup.inputFieldDisknameAndID("", sourceC64Dir.d64.diskName, sourceC64Dir.d64.diskID);
						if(diskNameAndID != null) {

							if(sourceMode == MODE_D64_DIR) {		

								sourceC64Dir.d64.setDiskNameAndID(diskNameAndID[0], diskNameAndID[1]);
								sourceC64Dir.reloadDir();								
							}
						
						}		
					}
					else {

						if(sourceMode == MODE_NORMAL_DIR) {  // .d64, .prg, .t64, .p00 

							String[] selectedFiles = sourceFileDir.getSelectedFiles();
							if(selectedFiles != null && selectedFiles.length == 1) {

								String selectedFilePath = sourceFileDir.directoryPath + "/" + selectedFiles[0];							
								if(selectedFilePath.toLowerCase().contains(".del") 
										|| selectedFilePath.toLowerCase().contains(".seq")
										|| selectedFilePath.toLowerCase().contains(".prg")
										|| selectedFilePath.toLowerCase().contains(".usr")
										|| selectedFilePath.toLowerCase().contains(".rel")
										|| selectedFilePath.toLowerCase().contains(".p00")
										|| selectedFilePath.toLowerCase().contains(".t64")	
										|| selectedFilePath.toLowerCase().contains(".d64")
										) {	

									File file = new File(selectedFilePath);

									String name = popup.inputField("Enter new name:", file.getName(), false);
									if(name.length() > 0) {

										file.renameTo(new File(file.getParentFile() + "/" + name));
										sourceFileDir.reloadDir();	
									}
								}
							}						
						}
						else if(sourceMode == MODE_D64_DIR) { // D64 File Names

							String[] selectedFiles = sourceC64Dir.getSelectedFiles();
							if(selectedFiles != null && selectedFiles.length == 1) {

								String selectedFile = selectedFiles[0];

								String name = popup.inputField("Enter new name:", selectedFile ,true);
								if(name.length() > 0) {

									DirEntry dirEntry = sourceC64Dir.d64.loadPRGFromDisk(selectedFile);	
									dirEntry.setFileName(name);
									sourceC64Dir.d64.writeDirEntryDataToDisk(dirEntry);

									sourceC64Dir.reloadDir();	
								}
							}
						}
						
					}
				} 
				catch (Exception e) {
					e.printStackTrace();
				}
			}
		} );
		T.start();	
	}

	public void deleteFile(int sourceSide) {

		Thread T = new Thread(new Runnable() {
			@Override
			public void run() {

				try {

					boolean reply = popup.question("Are you sure you want to Delete?");	
					if(!reply) return;

					SwingUtilities.invokeLater(new Runnable() {
						public void run() {							
							popup.hint("Deleting Files, please wait...");
						}});

					int sourceMode 			= leftDirMode;
					int targetMode 			= rightDirMode;									
					FileDir sourceFileDir 	= fileDirLeft;
					FileDir tragetFileDir 	= fileDirRight;
					C64Dir sourceC64Dir 		= c64DirLeft;

					if(sourceSide == 1) {

						sourceMode 		= rightDirMode;
						targetMode 		= leftDirMode;										
						sourceFileDir 	= fileDirRight;
						tragetFileDir 	= fileDirLeft;
						sourceC64Dir 	= c64DirRight;
					}

					if(sourceMode == MODE_NORMAL_DIR) {

						String[] selectedFiles = sourceFileDir.getSelectedFiles();
						for(int i=0;i<selectedFiles.length;i++) {

							String selectedFilePath = sourceFileDir.directoryPath + "/" + selectedFiles[i];							
							if(selectedFilePath.toLowerCase().contains(".d64") 
									|| selectedFilePath.toLowerCase().contains(".del") 
									|| selectedFilePath.toLowerCase().contains(".seq")
									|| selectedFilePath.toLowerCase().contains(".prg")
									|| selectedFilePath.toLowerCase().contains(".usr")
									|| selectedFilePath.toLowerCase().contains(".rel")
									|| selectedFilePath.toLowerCase().contains(".p00")
									|| selectedFilePath.toLowerCase().contains(".t64")	
									|| selectedFilePath.toLowerCase().contains(".g64")	
									|| selectedFilePath.toLowerCase().contains(".nib")	
									) {

								helper.deleteFile(new File(selectedFilePath));
								
								sourceFileDir.clearSelection();
								sourceFileDir.reloadDir();								
							}
						}
					}		

					if(sourceMode == MODE_D64_DIR) {

						String[] selectedFiles = sourceC64Dir.getSelectedFiles();
						if(selectedFiles != null && selectedFiles.length > 0) {

							for(int i=0;i<selectedFiles.length;i++) {

								DirEntry dirEntry = sourceC64Dir.d64.loadPRGFromDisk(selectedFiles[i]);	
								
								// * Files mit Filesize 0 nicht löschen! Da verschwindet das ganze Directory!!! *
								if(dirEntry.getFileBlockSize() > 0) sourceC64Dir.d64.deletePRGFromDisk(dirEntry);								
							}		

							sourceC64Dir.reloadDir();	
							sourceC64Dir.clearSelection();
						}
					}		

					
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							popup.close();		
						}});

				} 
				catch (Exception e) {e.printStackTrace();}
			}
		} );
		T.start();	
	}

	public void runEmu(int sourceSide) {

		int sourceMode 			= leftDirMode;
		FileDir sourceFileDir 	= fileDirLeft;
		C64Dir sourceC64Dir 		= c64DirLeft;

		if(sourceSide == 1) {

			sourceMode 		= rightDirMode;
			sourceFileDir 	= fileDirRight;
			sourceC64Dir 	= c64DirRight;
		}

		if(sourceMode == MODE_NORMAL_DIR) {  

			if(sourceFileDir.filePath.toLowerCase().contains(".d64") || sourceFileDir.filePath.toLowerCase().contains(".g64") || sourceFileDir.filePath.toLowerCase().contains(".nib") || sourceFileDir.filePath.toLowerCase().contains(".tap")) {

				try {	Desktop.getDesktop().open(new File(sourceFileDir.filePath));	} catch (Exception e) {	e.printStackTrace();	}	
			}
			else {

				try {	

					File tempFile = new File(internalPath + "/run.prg");
					if(tempFile.exists()) helper.deleteFile(tempFile);	

					helper.createTempPRG(sourceFileDir.filePath, tempFile.getAbsolutePath());
					if(tempFile.exists()) Desktop.getDesktop().open(tempFile);	
				} 
				catch (Exception e) {e.printStackTrace();}
			}
		}
		else if(sourceMode == MODE_D64_DIR) {  

			try {

				String[] selectedFiles = sourceC64Dir.getSelectedFiles();
				if(selectedFiles != null && selectedFiles.length > 0) {

					DirEntry dirEntry = sourceC64Dir.d64.loadPRGFromDisk(selectedFiles[0]);		
					byte[] data = sourceC64Dir.d64.getPRGData(dirEntry);	
					if(data != null && data.length > 0) {

						File tempFile = new File(internalPath + "/run.prg");
						if(tempFile.exists()) tempFile.delete();

						FileOutputStream fos = new FileOutputStream(tempFile);
						fos.write(data);
						fos.close(); 

						Desktop.getDesktop().open(tempFile);
					}
				}
			}
			catch(Exception e) {	e.printStackTrace();	}
		}
	
	}


	void copySelectedFiles(int sourceSide) {

		Thread T=new Thread(new Runnable() {

			@Override
			public void run() {

				try {

					//boolean reply = popup.question("Are you sure you want to COPY?");	
					//if(!reply) return;

					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							popup.hint("Copying Files, please wait...");
						}});

					int sourceMode 			= leftDirMode;
					int targetMode 			= rightDirMode;									
					FileDir sourceFileDir 	= fileDirLeft;
					FileDir tragetFileDir 	= fileDirRight;
					C64Dir sourceC64Dir 		= c64DirLeft;
					C64Dir targetC64Dir 		= c64DirRight;

					if(sourceSide == 1) {

						sourceMode 		= rightDirMode;
						targetMode 		= leftDirMode;										
						sourceFileDir 	= fileDirRight;
						tragetFileDir 	= fileDirLeft;
						sourceC64Dir 	= c64DirRight;
						targetC64Dir		= c64DirLeft;
					}

					boolean updateDirsAfterCommand = false;


					// *****************************************
					// **   LOCAL Directoy (prg d64)  Files   **
					// *****************************************					
					if(sourceMode == MODE_NORMAL_DIR) {  

						String[] selectedFiles = sourceFileDir.getSelectedFiles();
						for(int i=0;i<selectedFiles.length;i++) {

							String selectedFilePath = sourceFileDir.directoryPath + "/" + selectedFiles[i];							
							if(selectedFilePath.toLowerCase().contains(".del") 
									|| selectedFilePath.toLowerCase().contains(".seq")
									|| selectedFilePath.toLowerCase().contains(".prg")
									|| selectedFilePath.toLowerCase().contains(".usr")
									|| selectedFilePath.toLowerCase().contains(".rel")
									|| selectedFilePath.toLowerCase().contains(".p00")
									|| selectedFilePath.toLowerCase().contains(".t64")		
									|| selectedFilePath.toLowerCase().contains(".d64")	
									|| selectedFilePath.toLowerCase().contains(".g64")	
									|| selectedFilePath.toLowerCase().contains(".nib")	
									|| selectedFilePath.toLowerCase().contains(".bin")	
									|| selectedFilePath.toLowerCase().contains(".crt")	
									|| !selectedFilePath.toLowerCase().contains(".")	
									) {	


								boolean createdPRGFile = false; 																															

								if(selectedFilePath.toLowerCase().contains(".p00") || selectedFilePath.toLowerCase().contains(".t64")) {									

									String originalFileExt 	= helper.getExtension(new File(selectedFilePath));

									File tempFile = new File(internalPath + "/" + new File(selectedFilePath).getName().replace("." + originalFileExt, ".prg").toLowerCase());
									if(tempFile.exists()) tempFile.delete();
									helper.copyFile(new File(selectedFilePath), tempFile);

									helper.createTempPRG(selectedFilePath, tempFile.getAbsolutePath());
									createdPRGFile = true;

									selectedFilePath = tempFile.getAbsolutePath();
								}

								// * Copy .d64 File *

								if(selectedFilePath.toLowerCase().contains(".d64")) {	

									if(targetMode == MODE_D64_DIR) { 

										String extension = helper.getExtension(new File(targetC64Dir.directoryPath));
										String targetD64DiskName = new File(targetC64Dir.directoryPath).getName().replaceAll("."+extension, "");

										popup.close();
										boolean reply = popup.question("Are you sure you want to override Disk:\n\"" + targetD64DiskName+"."+extension+"\"");	
										if(!reply) return;

										SwingUtilities.invokeLater(new Runnable() {
											public void run() {
												popup.hint("Copying Disk, please wait...");
											}});


										updateDirsAfterCommand = true;
										Files.copy(new File(selectedFilePath).toPath(), new File(targetC64Dir.directoryPath).toPath(), StandardCopyOption.REPLACE_EXISTING);

									}	
									
								}
								else if(selectedFilePath.toLowerCase().contains(".g64") || selectedFilePath.toLowerCase().contains(".nib")) {	

								
								}
								else if(targetMode == MODE_D64_DIR) { 

									// * No more 144 Files on a Disk is allowed *
									if(targetC64Dir.d64.d64FileList.size() <= 135) {

										String originalFileExt 	= helper.getExtension(new File(selectedFilePath));
										String newFileNameInDir 	= new File(selectedFilePath).getName().replace("." + originalFileExt, "");

										newFileNameInDir 	= D64.reverseCase(newFileNameInDir);		

										if(targetC64Dir.d64.fileExist(newFileNameInDir)) {

											popup.close();
											popup.hintWithButton("\"" + newFileNameInDir + "\"\nstill exist!");
											continue;
										}

										int fileType = 0;
										if(originalFileExt.toLowerCase().compareTo("del") == 0) fileType = 0x80;
										else if(originalFileExt.toLowerCase().compareTo("seq") == 0) fileType = 0x81;
										else if(originalFileExt.toLowerCase().compareTo("prg") == 0) fileType = 0x82;
										else if(originalFileExt.toLowerCase().compareTo("usr") == 0) fileType = 0x83;
										else if(originalFileExt.toLowerCase().compareTo("rel") == 0) fileType = 0x84;
										else fileType = 0x82;
										
										byte[] prgFileData 	= readFileData(selectedFilePath);

										int errorCode = targetC64Dir.d64.writePRGData(newFileNameInDir.toUpperCase(), prgFileData, fileType);		
										if(errorCode == 0) 	updateDirsAfterCommand = true;
										else 				System.out.println("Error Code: " + errorCode);	

										if(createdPRGFile) helper.deleteFile(new File(selectedFilePath));
									}
								}									
								
							}							
							else {
								
								// * Copy other Files to other local Directory *
								
								if(targetMode == MODE_NORMAL_DIR) {	
									
									Files.copy(new File(selectedFilePath).toPath(), new File(tragetFileDir.directoryPath+ "/" + selectedFiles[i]).toPath(), StandardCopyOption.REPLACE_EXISTING);
									updateDirsAfterCommand = true;
								}								
							}
						}
					}

					// *****************************************
					// **      Copy D64 Directory Files       **
					// *****************************************					
					else if(sourceMode == MODE_D64_DIR) {

						if(targetMode == MODE_NORMAL_DIR) { 

							String[] selectedFiles = sourceC64Dir.getSelectedFiles();
							if(selectedFiles != null && selectedFiles.length > 0) {

								for(int i=0;i<selectedFiles.length;i++) {

									DirEntry dirEntry = sourceC64Dir.d64.loadPRGFromDisk(selectedFiles[i]);		
									byte[] data = sourceC64Dir.d64.getPRGData(dirEntry);	
									if(data != null && data.length > 0) {										

										String fileTypeExtension = "";

										if((dirEntry.getFileType() & 0x03) == 0) fileTypeExtension = "del";
										else if((dirEntry.getFileType() & 0x03) == 1) fileTypeExtension = "seq";
										else if((dirEntry.getFileType() & 0x03) == 2) fileTypeExtension = "prg";
										else if((dirEntry.getFileType() & 0x03) == 3) fileTypeExtension = "usr";
										else if((dirEntry.getFileType() & 0x03) == 4) fileTypeExtension = "rel";

										try (FileOutputStream fos = new FileOutputStream(tragetFileDir.directoryPath + "/" + selectedFiles[i].toLowerCase() + "." + fileTypeExtension)) {
											fos.write(data);
											fos.close(); 
											updateDirsAfterCommand = true;
										}
									}
								}								
							}
						}

						else if(targetMode == MODE_D64_DIR) { 

							// * No more 144 Files on a Disk is allowed *
							if(targetC64Dir.d64.d64FileList.size() <= 135) {

								String[] selectedFiles = sourceC64Dir.getSelectedFiles();
								if(selectedFiles != null && selectedFiles.length > 0) {

									for(int i=0;i<selectedFiles.length;i++) {

										if(targetC64Dir.d64.fileExist(selectedFiles[i])) {

											popup.close();
											popup.hintWithButton("\"" + selectedFiles[i] + "\"\nstill exist!");
											continue;
										}

										DirEntry dirEntry = sourceC64Dir.d64.loadPRGFromDisk(selectedFiles[i]);		
										byte[] data = sourceC64Dir.d64.getPRGData(dirEntry);	
										if(data != null) {

											updateDirsAfterCommand = true;											
											int errorCorde = targetC64Dir.d64.writePRGData(selectedFiles[i], data, dirEntry.getFileType());										
										}
									}								
								}
							}
						}

					}

					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							popup.close();		
						}});


					// * Update Directory if changed *
					if(updateDirsAfterCommand) updateDir(sourceSide);
				} 
				catch (Exception e) {e.printStackTrace();}
			}
		} );
		T.start();	
	}

	private byte[] readFileData(String path) {

		byte[] getBytes = {};

		try {

			File file = new File(path);
			getBytes = new byte[(int) file.length()];
			InputStream is = new FileInputStream(file);
			is.read(getBytes);
			is.close();
		} 
		catch (Exception e) {e.printStackTrace();} 

		return getBytes;
	}
}